@@ -186,7 +186,10 @@ static void redraw(void)
 
      XGetWindowAttributes(dpy, topLevel, &attr);
      buttonsize = attr.width/3;
-     if(attr.height < buttonsize) buttonsize=attr.height;
+     if(attr.height < buttonsize)
+     {
+          buttonsize=attr.height;
+     }
 
      if(old_buttonsize != buttonsize)
      {
@@ -205,15 +208,15 @@ static void redraw(void)
      /***** play ******/
 
      {
-         int s = (buttonDown != 0) ? d : 0;
+          int s = (buttonDown != 0) ? d : 0;
 
-         points[0] = coordAdd(&base,       scale(5)-s,  scale(2)-s);
-         points[1] = coordAdd(&points[0],  scale(2),    0);
-         points[2] = coordAdd(&points[1],  scale(5),    scale(5));
+          points[0] = coordAdd(&base,       scale(5)-s,  scale(2)-s);
+          points[1] = coordAdd(&points[0],  scale(2),    0);
+          points[2] = coordAdd(&points[1],  scale(5),    scale(5));
 
-         points[5] = coordAdd(&base,       scale(5)-s, scale(13)-s);
-         points[4] = coordAdd(&points[5],  scale(2),   0);
-         points[3] = coordAdd(&points[4],  scale(5),  -scale(5));
+          points[5] = coordAdd(&base,       scale(5)-s, scale(13)-s);
+          points[4] = coordAdd(&points[5],  scale(2),   0);
+          points[3] = coordAdd(&points[4],  scale(5),  -scale(5));
      }
 
      c = (state==STATE_PLAY) ? gc_onColor : gc_offColor;
@@ -231,15 +234,15 @@ static void redraw(void)
           /* Draw the shadow */
           for(i = BUTTON_DEPTH; i > 0; i--)
           {
-              XDrawLines(dpy, topLevel, gc_black, &points[2], 4, CoordModeOrigin);
+               XDrawLines(dpy, topLevel, gc_black, &points[2], 4, CoordModeOrigin);
 
-              XDrawLine(dpy, topLevel, gc_black, points[3].x-1, points[3].y, 
+               XDrawLine(dpy, topLevel, gc_black, points[3].x-1, points[3].y, 
                                                  points[4].x-1, points[4].y);
 
-              points[2].x++; points[2].y++;
-              points[3].x++; points[3].y++;
-              points[4].x++; points[4].y++;
-              points[5].x++; points[5].y++;
+               points[2].x++; points[2].y++;
+               points[3].x++; points[3].y++;
+               points[4].x++; points[4].y++;
+               points[5].x++; points[5].y++;
           }
      }
      else
@@ -299,7 +302,10 @@ static void redraw(void)
 static int igetenv(char *var, int def)
 {
      char *tmp=getenv(var);
-     if(!tmp) return def;
+     if(!tmp) 
+     {
+          return def;
+     }
      return atoi(tmp);
 }
 
@@ -340,24 +346,24 @@ static void my_play(char * command)
           }
           if(sig_fds[SIG_RD_FD] >= 0)
           {
-              close(sig_fds[SIG_RD_FD]);
+               close(sig_fds[SIG_RD_FD]);
           }
           if(sig_fds[SIG_WR_FD] >= 0)
           {
-              close(sig_fds[SIG_WR_FD]);
+               close(sig_fds[SIG_WR_FD]);
           }
           close(pipe_fd);
 
           /* Redirect stdout & stderr to /dev/null */
           if(flags & H_NOISY)
           {
-                const int ofd = open("/dev/null", O_RDONLY);
-                if(ofd)
-                {
-                     dup2(ofd, 1);
-                     dup2(ofd, 2);
-                     close(ofd);
-                }
+               const int ofd = open("/dev/null", O_RDONLY);
+               if(ofd)
+               {
+                    dup2(ofd, 1);
+                    dup2(ofd, 2);
+                    close(ofd);
+               }
           }
 
           setpgid(pid, 0);
@@ -373,9 +379,13 @@ static void my_play(char * command)
      {
           state=STATE_PLAY;
           if(!repeats) 
-              repeats = repeatsResetVal;
+          {
+               repeats = repeatsResetVal;
+          }
           if((repeats > 0) && (repeats != MAXINT)) 
-              repeats--;
+          {
+               repeats--;
+          }
      }
 }
 
@@ -393,10 +403,13 @@ static void my_pause(char * command)
      if(state != STATE_STOP)
      {
           if(!kill(-pid, SIGSTOP))
+          {
                state = STATE_PAUSE;
+          }
           else
+          {
                state = STATE_STOP;
-
+          }
           return;
      }
 }
@@ -410,7 +423,10 @@ static void my_pause(char * command)
  *****************************************************************************/
 static void low_die(void)
 {
-     if(pid > 0) my_kill(-pid);
+     if(pid > 0)
+     {
+          my_kill(-pid);
+     }
      _exit(0);
 }
 
@@ -425,10 +441,16 @@ static void low_die(void)
  *****************************************************************************/
 static void my_stop(char * command)
 {
-     if(state == STATE_PAUSE) my_play(command);
+     if(state == STATE_PAUSE) 
+     {
+          my_play(command);
+     }
      if(state == STATE_PLAY)
      {
-          if(pid > 0) my_kill(-pid);
+          if(pid > 0) 
+          {
+               my_kill(-pid);
+          }
           state=STATE_STOP;
           repeats=0;
      }
@@ -499,11 +521,11 @@ static void sigdie(int sig)
  *****************************************************************************/
 static void sig_to_fd(int sig)
 {
-    D("Signal %i routed to pipe\n", sig);
-    if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
-    {
-        D("Signal write to pipe failed\n");
-    }
+     D("Signal %i routed to pipe\n", sig);
+     if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
+     {
+          D("Signal write to pipe failed\n");
+     }
 }
 
 /******************************************************************************/
@@ -558,7 +580,9 @@ static void check_x_events(char * command)
 
           case Expose:
                if(ev.xexpose.count)
+               {
                     break;
+               }
           case ResizeRequest:
           case MapNotify:
                redraw();
@@ -596,40 +620,40 @@ static void check_x_events(char * command)
 static void normalise_window_coords(unsigned * width, unsigned * height,
                                     int * x, int * y)
 {
-    unsigned w = *width;
-    unsigned h = *height;
-    const unsigned target_aspect = DEFAULT_CONTROLS_WIDTH /
+     unsigned w = *width;
+     unsigned h = *height;
+     const unsigned target_aspect = DEFAULT_CONTROLS_WIDTH /
                                                DEFAULT_CONTROLS_HEIGHT;
-    if(smallControls)
-    {
-        *y = *x = 0;
-        *width  = MIN_CONTROLS_WIDTH;
-        *height = MIN_CONTROLS_HEIGHT;
-        return;
-    }
+     if(smallControls)
+     {
+          *y = *x = 0;
+          *width  = MIN_CONTROLS_WIDTH;
+          *height = MIN_CONTROLS_HEIGHT;
+          return;
+     }
 
-    if(flags & H_FILL)        /* Dont bother if user wants to fill the window */
-    {
-        *x = *y = 0;
-        return;
-    }
+     if(flags & H_FILL)        /* Dont bother if user wants to fill the window */
+     {
+          *x = *y = 0;
+          return;
+     }
 
-    if((w > MAX_CONTROLS_WIDTH) & !(flags & H_MAXASPECT))
-    {
-        w = MAX_CONTROLS_WIDTH;
-    }
+     if((w > MAX_CONTROLS_WIDTH) & !(flags & H_MAXASPECT))
+     {
+          w = MAX_CONTROLS_WIDTH;
+     }
 
-    /* Keep the controls as close to default ratio as possible */
-    if(h > w / target_aspect)
-    {
-        h = w / target_aspect;
-    }
+     /* Keep the controls as close to default ratio as possible */
+     if(h > w / target_aspect)
+     {
+          h = w / target_aspect;
+     }
 
-    *x = (int) (*width - w)/2;
-    *y = (int) (*height - h)/2;
+     *x = (int) (*width - w)/2;
+     *y = (int) (*height - h)/2;
 
-    *height = h;
-    *width = w;
+     *height = h;
+     *width = w;
 }
 
 /******************************************************************************/
@@ -653,7 +677,9 @@ static void check_pipe_fd_events(void)
      if (n < 0)
      {
           if (errno == EINTR)
+          {
                return;
+          }
           exit(EX_UNAVAILABLE);
      }
   
@@ -882,7 +908,9 @@ int main(int argc, char **argv)
      }
 
      if(igetenv("autostart",1))
+     {
           my_play(command);
+     }
 
      while(1)
      {
@@ -919,8 +947,8 @@ int main(int argc, char **argv)
                }
                if(FD_ISSET(sig_fds[SIG_RD_FD], &fds))
                {
-                   int sig;
-                   read(sig_fds[SIG_RD_FD], &sig, sizeof(int));
+                    int sig;
+                    read(sig_fds[SIG_RD_FD], &sig, sizeof(int));
                }
           }
           D("SELECT OUT\n");
@@ -934,7 +962,10 @@ int main(int argc, char **argv)
                     if(state == STATE_PLAY)
                     {
                          state = STATE_STOP;
-                         if(repeats > 0) my_play(command);
+                         if(repeats > 0) 
+                         {
+                              my_play(command);
+                         }
                     }
                }
           }
