@@ -61,9 +61,9 @@ static struct
 
 static struct 
 {
-    Window  window;
-    int     width;
-    int     height;
+     Window  window;
+     int     width;
+     int     height;
 } parentDetails;
 
 
@@ -132,15 +132,15 @@ static int error_handler(Display *dpy, XErrorEvent *err)
  *****************************************************************************/
 static void initSwallowMutex(void)
 {
-    D("Initialising Swallow Mutex\n"); 
-    if(!display) 
-    {
-	fprintf(stderr, "Display not set so cannot initialise semaphore!\n");
-        return;
-    }
+     D("Initialising Swallow Mutex\n"); 
+     if(!display) 
+     {
+	  fprintf(stderr, "Display not set so cannot initialise semaphore!\n");
+          return;
+     }
 
-    swallowMutex = XInternAtom(display, "MOZPLUGGER_SWALLOW_MUTEX", 0);
-    swallowMutexTaken = 0;
+     swallowMutex = XInternAtom(display, "MOZPLUGGER_SWALLOW_MUTEX", 0);
+     swallowMutexTaken = 0;
 }
 #endif
 
@@ -153,7 +153,7 @@ static void initSwallowMutex(void)
  *****************************************************************************/
 static void initWindowOwnerMarker(void)
 {
-    windowOwnerMark = XInternAtom(display, "MOZPLUGGER_OWNER", 0);
+     windowOwnerMark = XInternAtom(display, "MOZPLUGGER_OWNER", 0);
 }
 
 /*****************************************************************************/
@@ -165,27 +165,27 @@ static void initWindowOwnerMarker(void)
  * @return The host ID
  *
  *****************************************************************************/
-static uint32 getHostId(void)
+static uint32_t getHostId(void)
 {
-    char hostName[128];
-    uint32 id;
-    int i;
+     char hostName[128];
+     uint32_t id;
+     int i;
    
-    memset(hostName, 0, sizeof(hostName));
-    gethostname(hostName, sizeof(hostName)-1);
+     memset(hostName, 0, sizeof(hostName));
+     gethostname(hostName, sizeof(hostName)-1);
 
-    D("Host Name = \"%s\"\n", hostName);
+     D("Host Name = \"%s\"\n", hostName);
 
-    /* OK, Create a 32 bit hash value from the host name....
+     /* OK, Create a 32 bit hash value from the host name....
        use this as a host ID, the possiblity of a collison of hash keys 
        effecting the swallow of victims is so infinitimisally small! */
 
-    id = 0;
-    for(i=0; i < (int)(sizeof(hostName)/sizeof(uint32)); i++)
-    {
-        id = ((id << 5) ^ (id >> 27)) ^ ((uint32 *)hostName)[i];
-    }
-    return id;
+     id = 0;
+     for(i=0; i < (int)(sizeof(hostName)/sizeof(uint32_t)); i++)
+     {
+          id = ((id << 5) ^ (id >> 27)) ^ ((uint32_t *)hostName)[i];
+     }
+     return id;
 }
 
 /*****************************************************************************/
@@ -203,42 +203,42 @@ static uint32 getHostId(void)
  * @return 1(true) if owner found, 0 otherwise
  *
  *****************************************************************************/
-static int getOwnerFromProperty(Window w, Atom name, uint32 * hostId, uint32 * pid)
+static int getOwnerFromProperty(Window w, Atom name, uint32_t * hostId, uint32_t * pid)
 {
-    unsigned long nitems;
-    unsigned long bytes;      
-    int fmt;
-    Atom type;
-    unsigned char * property = NULL;
-    int success = 0;
+     unsigned long nitems;
+     unsigned long bytes;      
+     int fmt;
+     Atom type;
+     unsigned char * property = NULL;
+     int success = 0;
 
-    /* Get hold of the Host & PID that current holds the semaphore for 
+     /* Get hold of the Host & PID that current holds the semaphore for 
        this display! - watch out for the bizarre 64-bit platform behaviour
        where property returned is actually an array of 2 x  64bit ints with
        the top 32bits set to zero! */
-    XGetWindowProperty(display, w, name,
+     XGetWindowProperty(display, w, name,
                        0, 2, 0, XA_INTEGER,
                        &type, &fmt, &nitems, &bytes,
                        &property);
 
-    if(property)
-    {
-        /* Just check all is correct! */
-	if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2))
-        {
-	    fprintf(stderr, "XGetWindowProperty returned bad values " 
+     if(property)
+     {
+          /* Just check all is correct! */
+	  if((type != XA_INTEGER) || (fmt!=32) || (nitems!=2))
+          {
+	       fprintf(stderr, "XGetWindowProperty returned bad values " 
                             "%ld,%d,%lu,%lu\n", (long) type, fmt, nitems, 
                             bytes);
-        }
-	else
-	{
-            *hostId = (uint32)((unsigned long *)property)[0];
-            *pid = (uint32)((unsigned long *)property)[1];
-            success = 1;
-        }
-	XFree(property);
-    }
-    return success;
+          }
+	  else
+	  {
+               *hostId = (uint32_t)((unsigned long *)property)[0];
+               *pid = (uint32_t)((unsigned long *)property)[1];
+               success = 1;
+          }
+	  XFree(property);
+     }
+     return success;
 }
 
 #ifdef USE_MUTEX_LOCK
@@ -254,9 +254,9 @@ static int getOwnerFromProperty(Window w, Atom name, uint32 * hostId, uint32 * p
  * @return 1(true) if owner found, 0 otherwise
  *
  *****************************************************************************/
-static int getSwallowMutexOwner(uint32 * hostId, uint32 * pid)
+static int getSwallowMutexOwner(uint32_t * hostId, uint32_t * pid)
 {   
-    return getOwnerFromProperty(wattr.root, swallowMutex, hostId, pid);
+     return getOwnerFromProperty(wattr.root, swallowMutex, hostId, pid);
 }
 
 /*****************************************************************************/
@@ -269,13 +269,13 @@ static int getSwallowMutexOwner(uint32 * hostId, uint32 * pid)
  * @param[in] pid    The process ID of the mutex
  *
  *****************************************************************************/
-static void setSwallowMutexOwner(uint32 hostId, uint32 pid)
+static void setSwallowMutexOwner(uint32_t hostId, uint32_t pid)
 {
-    unsigned long temp[2] = {hostId, pid};
-    D("Setting swallow mutex owner, hostId = 0x%08X, pid=%u\n", 
+     unsigned long temp[2] = {hostId, pid};
+     D("Setting swallow mutex owner, hostId = 0x%08X, pid=%u\n", 
                                             (unsigned) hostId, (unsigned) pid); 
 
-    XChangeProperty(display, wattr.root, swallowMutex,
+     XChangeProperty(display, wattr.root, swallowMutex,
                         XA_INTEGER, 32, PropModeAppend,
                         (unsigned char*) (&temp), 2);
 }
@@ -289,74 +289,76 @@ static void setSwallowMutexOwner(uint32 hostId, uint32 pid)
  *****************************************************************************/
 static void takeSwallowMutex(void)
 {
-    int countDown;
-    const uint32 ourPid = (uint32)getpid();
-    const uint32 ourHostId = getHostId();
-    uint32 otherPid;
-    uint32 otherHostId;
-    uint32 prevOtherPid = 0;
+     int countDown;
+     const uint32_t ourPid = (uint32_t)getpid();
+     const uint32_t ourHostId = getHostId();
+     uint32_t otherPid;
+     uint32_t otherHostId;
+     uint32_t prevOtherPid = 0;
  
-    if((display == 0) || (swallowMutex == 0))
-	return;
+     if((display == 0) || (swallowMutex == 0))
+     {
+	  return;
+     }
 
-    D("Attempting to take Swallow Mutex\n"); 
-    /* Try up tp forty times ( 40 * 250ms = 10 seconds) to take 
+     D("Attempting to take Swallow Mutex\n"); 
+     /* Try up tp forty times ( 40 * 250ms = 10 seconds) to take 
        the semaphore... */
 
-    countDown = 40;
-    while(1)
-    {
+     countDown = 40;
+     while(1)
+     {
 
-        /* While someone owns the semaphore */
-        while(getSwallowMutexOwner(&otherHostId, &otherPid))
-        {
-            if( otherHostId == ourHostId)    
-            {
-                if(otherPid == ourPid)     
-                {
- 	            /* Great we have either successfully taken the semaphore
-                       OR (unlikely) we previously had the semaphore! Exit
-                       the function...*/
-	            swallowMutexTaken = 1;
-                    D("Taken Swallow Mutex\n"); 
-		    return;
-                }
+          /* While someone owns the semaphore */
+          while(getSwallowMutexOwner(&otherHostId, &otherPid))
+          {
+               if( otherHostId == ourHostId)    
+               {
+                    if(otherPid == ourPid)     
+                    {
+ 	                /* Great we have either successfully taken the semaphore
+                           OR (unlikely) we previously had the semaphore! Exit
+                           the function...*/
+	                 swallowMutexTaken = 1;
+                         D("Taken Swallow Mutex\n"); 
+		         return;
+                    }
 
-     	    	D("Semaphore currently taken by pid=%ld\n", otherPid);
- 		/* Check that the process that has the semaphore exists...
-        	   Bit of a hack, I cant find a function to directly check
-                   if process exists. */
-            	if( (getsid(otherPid) < 0 ) && (errno == ESRCH))
-	        {
-	    	    D("Strange other Pid(%lu) cannot be found\n", otherPid);
-                    XDeleteProperty(display, wattr.root, swallowMutex);
-                    break;    /* OK force early exit of inner while loop */
-                }
-	    }
+     	    	    D("Semaphore currently taken by pid=%ld\n", (long) otherPid);
+ 		    /* Check that the process that has the semaphore exists...
+        	     Bit of a hack, I cant find a function to directly check
+                     if process exists. */
+            	    if( (getsid(otherPid) < 0 ) && (errno == ESRCH))
+	            {
+	    	         D("Strange other Pid(%lu) cannot be found\n",(unsigned long) otherPid);
+                         XDeleteProperty(display, wattr.root, swallowMutex);
+                         break;    /* OK force early exit of inner while loop */
+                    }
+	       }
 
-            /* Check if the owner of semaphore hasn't recently changed if it
-               has restart timer */
-            if(prevOtherPid != otherPid)
-            {
-   	        D("Looks like semaphore's owner has changed pid=%ld\n", 
-                     otherPid);
-	        countDown = 40;
-                prevOtherPid = otherPid;
-            }
+               /* Check if the owner of semaphore hasn't recently changed if it
+                has restart timer */
+               if(prevOtherPid != otherPid)
+               {
+   	            D("Looks like semaphore's owner has changed pid=%ld\n", 
+                        (long) otherPid);
+	            countDown = 40;
+                    prevOtherPid = otherPid;
+               }
             
-            /* Do one step of the timer... */
-	    countDown--;
-            if(countDown <= 0)
-            {
-	    	D("Waited long enough for Pid(%lu)\n", otherPid);
-                XDeleteProperty(display, wattr.root, swallowMutex);
-		break;
-            }
+               /* Do one step of the timer... */
+	       countDown--;
+               if(countDown <= 0)
+               {
+	    	    D("Waited long enough for Pid(%lu)\n", (unsigned long) otherPid);
+                    XDeleteProperty(display, wattr.root, swallowMutex);
+		    break;
+               }
 	
-            usleep(250000);        /* 250ms */
- 	}
+               usleep(250000);        /* 250ms */
+ 	  }
 
-        /* else no one has semaphore, timeout, or owner is dead -
+          /* else no one has semaphore, timeout, or owner is dead -
            Set us as the owner, but we need to check if we weren't 
            beaten to it so once more around the loop.
            Note even doing the recheck does not work in 100% of all
@@ -364,8 +366,8 @@ static void takeSwallowMutex(void)
            see Mozdev bug 20088 - the fix is done use the stillHaveMutex
            function */
 
-        setSwallowMutexOwner(ourHostId, ourPid);
-    }
+          setSwallowMutexOwner(ourHostId, ourPid);
+     }
 }
 
 /*****************************************************************************/
@@ -377,22 +379,23 @@ static void takeSwallowMutex(void)
  *****************************************************************************/
 static int stillHaveMutex(void)
 {
-     uint32 otherPid;
-     uint32 otherHostId;
+     uint32_t otherPid;
+     uint32_t otherHostId;
 
      if(getSwallowMutexOwner(&otherHostId, &otherPid))
      {
-          const uint32 ourHostId = getHostId();
+          const uint32_t ourHostId = getHostId();
           if( otherHostId == ourHostId)    
           {
-               const uint32 ourPid = (uint32)getpid();
+               const uint32_t ourPid = (uint32_t)getpid();
                if(otherPid == ourPid)     
                {
                     return 1;
                }
           }
      }
-     D("Race condition detected, semaphore pinched by Pid(%lu)\n", otherPid);
+     D("Race condition detected, semaphore pinched by Pid(%lu)\n", 
+                                                      (unsigned long) otherPid);
      return 0;
 }
  
@@ -405,12 +408,13 @@ static int stillHaveMutex(void)
  *****************************************************************************/
 static void giveSwallowMutex()
 {
-    if((display == 0) || (swallowMutex == 0) || (swallowMutexTaken ==0))
-	return;
-
-    D("Giving Swallow Mutex\n"); 
-    XDeleteProperty(display, wattr.root, swallowMutex);
-    swallowMutexTaken = 0;
+     if((display == 0) || (swallowMutex == 0) || (swallowMutexTaken ==0))
+     {
+	  return;
+     }
+     D("Giving Swallow Mutex\n"); 
+     XDeleteProperty(display, wattr.root, swallowMutex);
+     swallowMutexTaken = 0;
 }
 #endif
 
@@ -430,11 +434,11 @@ static void giveSwallowMutex()
 static int chkAndMarkVictimWindow(Window w)
 {
      unsigned long temp[2];
-     uint32 ourPid;
-     uint32 ourHostId;
+     uint32_t ourPid;
+     uint32_t ourHostId;
 
-     uint32 pid;
-     uint32 hostId;
+     uint32_t pid;
+     uint32_t hostId;
      int gotIt = 0;
 
      /* See if some other instance of Mozplugger has already 'marked' this
@@ -445,7 +449,7 @@ static int chkAndMarkVictimWindow(Window w)
      }
 
      /* OK lets claim it for ourselves... */
-     ourPid = (uint32)getpid();
+     ourPid = (uint32_t)getpid();
      ourHostId = getHostId();
 
      temp[0] = ourHostId;
@@ -465,8 +469,8 @@ static int chkAndMarkVictimWindow(Window w)
      }
      else         
      {
-         D("Strange, couldn't set windowOwnerMark\n");
-         gotIt = 1;
+          D("Strange, couldn't set windowOwnerMark\n");
+          gotIt = 1;
      }
 
      return gotIt;
@@ -486,8 +490,14 @@ static int chkAndMarkVictimWindow(Window w)
  *****************************************************************************/
 static int gcd(int a, int b)
 {
-     if (a < b) return gcd(b,a);
-     if (b == 0) return a;
+     if (a < b) 
+     {
+          return gcd(b,a);
+     }
+     if (b == 0) 
+     {
+          return a;
+     }
      return gcd(b, a % b);
 }
 
@@ -528,9 +538,11 @@ static void adjust_window_size(void)
      int w = parentDetails.width;
      int h = parentDetails.height;
 
-     int tmpw, tmph;
 
-     if (!victimDetails.window) return;
+     if (!victimDetails.window) 
+     {
+          return;
+     }
      
      if (!victimDetails.mapped || !victimDetails.reparented)
      {
@@ -546,11 +558,16 @@ static void adjust_window_size(void)
      {
 	  if (xaspect && yaspect)
 	  {
+               int tmpw, tmph;
+
 	       D("Resizing window %x with MAXASPECT\n", 
                                                (unsigned) victimDetails.window);
 	       tmph = h / yaspect;
 	       tmpw = w / xaspect;
-	       if (tmpw < tmph) tmph = tmpw;
+	       if (tmpw < tmph) 
+               {
+                    tmph = tmpw;
+               }
 	       tmpw = tmph * xaspect;
 	       tmph = tmph * yaspect;
 	       
@@ -644,9 +661,14 @@ static void change_leader(void)
  *****************************************************************************/
 static int reparent_window(void)
 {
-     if (!victimDetails.window) return 0;
-     if (!parentDetails.window) return 0;
-
+     if (!victimDetails.window) 
+     {
+          return 0;
+     }
+     if (!parentDetails.window) 
+     {
+          return 0;
+     }
      if(victimDetails.reparentedAttemptCount > 10)
      {
           D("Giving up reparenting to avoid - tried 10 times\n");
@@ -714,8 +736,14 @@ static int is_viewable(Display *display, Window w)
  *****************************************************************************/
 static int my_strcmp(char *windowname, char *name)
 {
-     if (!name) return 1;
-     if (!windowname) return 1;
+     if (!name) 
+     {
+          return 1;
+     }
+     if (!windowname) 
+     {
+          return 1;
+     }
 
      switch (name[0])
      {
@@ -854,8 +882,10 @@ static Window find_winrecur(Window from, int depth, char *name)
      /* This window doesn't match our 'HIDDEN' criteria, ignore it
       * (and its children) */
      if (!(flags & H_HIDDEN) && !is_viewable(display, from))
+     {
 	  return (Window)0;
-  
+     }
+
      if (check_window_name(from, name))
      {
           for (e=0; e < ignore_window_count; e++) 
@@ -885,8 +915,8 @@ static Window find_winrecur(Window from, int depth, char *name)
           if (!XQueryTree(display, from, &root, &parent, 
                                          &children, &num_children))
           {
-	        D("find_winrecur: Querytree failed!!!\n");
-	        return (Window)0;
+	       D("find_winrecur: Querytree failed!!!\n");
+	       return (Window)0;
           }
 
 	  D("find_winrecur: Num children = %d\n", num_children);
@@ -1122,9 +1152,9 @@ static void check_x_events(void)
 		    break;
 
                default: 
-                   /* 21 = ReparentNotify */
-	           D("!!Got unhandled event for root->%d\n", ev.type);
-                   break;
+                    /* 21 = ReparentNotify */
+	            D("!!Got unhandled event for root->%d\n", ev.type);
+                    break;
 	       }
 	  }
 	  else if (victimDetails.window 
@@ -1172,9 +1202,9 @@ static void check_x_events(void)
                     break;
 
                default: 
-                   /* 22 = ConfigureNotify, 33 = ClientMessage */
-	           D("!!Got unhandled event for victim->%d\n", ev.type);
-                   break;
+                    /* 22 = ConfigureNotify, 33 = ClientMessage */
+	            D("!!Got unhandled event for victim->%d\n", ev.type);
+                    break;
 	       }
 	  }
           else if (parentDetails.window 
@@ -1257,7 +1287,10 @@ static void check_pipe_fd_events(void)
      n = read(pipe_fd, ((char *)& wintmp),  sizeof(wintmp));
      if (n < 0)
      {
-	  if (errno == EINTR) return;
+	  if (errno == EINTR) 
+          {
+               return;
+          }
 	  D("Winddata read error, exiting\n");
 #ifdef USE_MUTEX_LOCK
           giveSwallowMutex();
@@ -1275,7 +1308,9 @@ static void check_pipe_fd_events(void)
      }
   
      if (n != sizeof(wintmp))
+     {
 	  return;
+     }
   
      parentDetails.window = (Window) wintmp.window;
      parentDetails.width = (int) wintmp.width;
@@ -1324,26 +1359,26 @@ static void check_all_events(pid_t pid)
 
           if(sig_fds[SIG_RD_FD] >= 0)
           {
-              FD_SET(sig_fds[SIG_RD_FD], &fds);
-              maxfd = MAX(maxfd, sig_fds[SIG_RD_FD]);
+               FD_SET(sig_fds[SIG_RD_FD], &fds);
+               maxfd = MAX(maxfd, sig_fds[SIG_RD_FD]);
           }
      
           D("SELECT IN\n");
           if (select(maxfd + 1, &fds, NULL, NULL, 0) > 0)
           {
-	      if (FD_ISSET(pipe_fd, &fds))
-              {
-	           check_pipe_fd_events();
-              }
-              if( FD_ISSET(sig_fds[SIG_RD_FD], &fds))
-              {
-                  int sig;
-                  read(sig_fds[SIG_RD_FD], &sig, sizeof(sig));
-              }  
+	       if (FD_ISSET(pipe_fd, &fds))
+               {
+	            check_pipe_fd_events();
+               }
+               if( FD_ISSET(sig_fds[SIG_RD_FD], &fds))
+               {
+                    int sig;
+                    read(sig_fds[SIG_RD_FD], &sig, sizeof(sig));
+               }  
           }
           else
           {
-              D("Select exited unexpectly, errno = %i\n", errno);
+               D("Select exited unexpectly, errno = %i\n", errno);
           }
           D("SELECT OUT\n");
 
@@ -1375,11 +1410,11 @@ static void check_all_events(pid_t pid)
  *****************************************************************************/
 static void sig_to_fd(int sig)
 {
-    D("Signal %i routed to pipe\n", sig);
-    if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
-    {
-        D("Signal write to pipe failed\n");
-    }
+     D("Signal %i routed to pipe\n", sig);
+     if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
+     {
+          D("Signal write to pipe failed\n");
+     }
 }
 
 /******************************************************************************/
@@ -1446,6 +1481,73 @@ static void exitEarly(void)
      exit(1);
 }
 
+/******************************************************************************/
+/**
+ * Expands the winname if it contains %f or %p by replacing %f by the file
+ * name part of the URL or %p by the whole of thr URL.
+ *
+ *****************************************************************************/
+void expand_winname(char * buffer, int bufferLen, const char * winame, const char * file)
+{
+     const char * p = winame;
+     char * q = buffer;
+     char * end = &buffer[bufferLen-1];
+
+     D("winame before expansion is '%s'\n", winname);
+
+     while((*p != '\0') && (q < end))
+     {
+          if( *p != '%')
+          {
+               *q++ = *p++;
+          }
+          else
+          {
+               const char * r;
+               switch(p[1])
+               {
+                    case '%' :
+                         *q++ = '%';
+                         p += 2;
+                         break;
+
+                    case 'f' :       /* %f = insert just the filename no path */
+                         r = strrchr(file, '/');   /* Find the filename start */
+                         if(r == 0)
+                         {
+                              r = file;
+                         }
+                         else
+                         {
+                              r++;       /* Skip the slash */
+                         }
+                         while((*r != '\0') && (q < end))
+                         {
+                              *q++ = *r++;
+                         }
+                         p += 2;       /* Skip the %f */ 
+                         break;
+
+                    case 'p' :
+                         r = file;
+                         while((*r != '\0') && (q < end))
+                         {
+                              *q++ = *r++;
+                         }
+                         p += 2;
+                         break;
+
+                    default :
+                         *q++ = *p++;
+                         break;
+               }
+          }
+     }
+     *q = '\0';
+
+     D("winame after expansion is '%s'\n", buffer);
+}
+
 /******************************************************************************
  * @brief main() 
  *
@@ -1459,6 +1561,8 @@ static void exitEarly(void)
  *****************************************************************************/
 int main(int argc, char **argv)
 {
+     char buffer[100];
+
      unsigned long temp = 0;
      int x, y, i;
      D("Helper started.....\n");
@@ -1492,6 +1596,11 @@ int main(int argc, char **argv)
      winname = getenv("winname");
      file = getenv("file");
 
+     if(winname)
+     {
+          expand_winname(buffer, sizeof(buffer), winname, file);
+          winname = buffer;
+     }
      D("HELPER: %s %s %s %s\n",
        argv[0],
        argv[1],
@@ -1512,7 +1621,10 @@ int main(int argc, char **argv)
           signal(SIGCHLD, sig_to_fd);
      }
 
-     if (repeats < 1) repeats = 1;
+     if (repeats < 1) 
+     {
+          repeats = 1;
+     }
 
      while (repeats > 0)
      {
@@ -1522,11 +1634,15 @@ int main(int argc, char **argv)
 
 	  /* This application will use the $repeat variable */
 	  if (flags & H_REPEATCOUNT)
+          {
 	       loops = repeats;
+          }
 
 	  /* Expecting the application to loop */
 	  if (flags & H_LOOP)
+          {
 	       loops = MAXINT;
+          }
 
 	  /* Create command line */
 	  app_argv[0] = "/bin/sh";
@@ -1601,7 +1717,9 @@ int main(int argc, char **argv)
 	       handle_app(pid);
 	       D("Wait done (repeats=%d, loops=%d)\n", repeats, loops);
 	       if (repeats < MAXINT)
+               {
 		    repeats -= loops;
+               }
 	  }
           /* On each repeat we re-open the XServer connection, so need to
            * close it at end of loop - TODO perhaps we should keep in open
