@@ -68,11 +68,11 @@ static int sig_fds[2];
 
 static struct 
 {
-    Window  window;
-    int x;
-    int y;
-    unsigned int width;
-    unsigned int height;
+     Window  window;
+     int x;
+     int y;
+     unsigned int width;
+     unsigned int height;
 } parentDetails;
 
 static GC gc_white;
@@ -139,7 +139,10 @@ static void redraw(void)
 
      XGetWindowAttributes(dpy, topLevel, &attr);
      buttonsize = attr.width;
-     if(attr.height < buttonsize) buttonsize=attr.height;
+     if(attr.height < buttonsize) 
+     {
+          buttonsize=attr.height;
+     }
 
      if(old_buttonsize != buttonsize)
      {
@@ -173,15 +176,15 @@ static void redraw(void)
           /* Draw the shadow */
           for(i = BUTTON_DEPTH; i > 0; i--)
           {
-              XDrawLines(dpy, topLevel, gc_black, &points[2], 4, CoordModeOrigin);
+               XDrawLines(dpy, topLevel, gc_black, &points[2], 4, CoordModeOrigin);
 
-              XDrawLine(dpy, topLevel, gc_black, points[3].x-1, points[3].y, 
+               XDrawLine(dpy, topLevel, gc_black, points[3].x-1, points[3].y, 
                                                  points[4].x-1, points[4].y);
 
-              points[2].x++; points[2].y++;
-              points[3].x++; points[3].y++;
-              points[4].x++; points[4].y++;
-              points[5].x++; points[5].y++;
+               points[2].x++; points[2].y++;
+               points[3].x++; points[3].y++;
+               points[4].x++; points[4].y++;
+               points[5].x++; points[5].y++;
           }
      }
      else
@@ -220,11 +223,11 @@ static void my_start(char * command)
      /* If another helper application, switch to that one */
      if(nextHelper)
      {
-         char buffer[ENV_BUFFER_SIZE];
-         char *cmd[4];
+          char buffer[ENV_BUFFER_SIZE];
+          char *cmd[4];
 
-         XDestroyWindow(dpy, topLevel);
-         close(ConnectionNumber(dpy));
+          XDestroyWindow(dpy, topLevel);
+          close(ConnectionNumber(dpy));
 
           if(sig_fds[0] >= 0)
           {
@@ -236,9 +239,9 @@ static void my_start(char * command)
                close(sig_fds[1]);
           }
 
-         cmd[0] = (char *) nextHelper;
+          cmd[0] = (char *) nextHelper;
 
-         snprintf(buffer, sizeof(buffer), "%d,%d,%d,%lu,%d,%d,%d,%d",
+          snprintf(buffer, sizeof(buffer), "%d,%d,%d,%lu,%d,%d,%d,%d",
                   flags,
                   repeats,
                   pipe_fd,
@@ -249,13 +252,13 @@ static void my_start(char * command)
                   (int) parentDetails.height);
 
 
-         cmd[1] = buffer;
-         cmd[2] = command;
-         cmd[3] = 0;
+          cmd[1] = buffer;
+          cmd[2] = command;
+          cmd[3] = 0;
 
-         D("Switching to helper\n");
-         execvp(nextHelper, cmd);
-         exit(EX_UNAVAILABLE);
+          D("Switching to helper\n");
+          execvp(nextHelper, cmd);
+          exit(EX_UNAVAILABLE);
      }
 
      /* else linker is used to launch command */
@@ -286,13 +289,13 @@ static void my_start(char * command)
           /* Redirect stdout & stderr to /dev/null */
           if(flags & H_NOISY)
           {
-                const int ofd = open("/dev/null", O_RDONLY);
-                if(ofd)
-                {
-                     dup2(ofd, 1);
-                     dup2(ofd, 2);
-                     close(ofd);
-                }
+               const int ofd = open("/dev/null", O_RDONLY);
+               if(ofd)
+               {
+                    dup2(ofd, 1);
+                    dup2(ofd, 2);
+                    close(ofd);
+               }
           }
 
           setpgid(pid, 0);
@@ -307,9 +310,13 @@ static void my_start(char * command)
      else
      {
           if(!repeats) 
-              repeats = repeatsResetVal;
+          {
+               repeats = repeatsResetVal;
+          }
           if((repeats > 0) && (repeats != MAXINT)) 
-              repeats--;
+          {
+               repeats--;
+          }
      }
 }
 
@@ -322,7 +329,10 @@ static void my_start(char * command)
  *****************************************************************************/
 static void low_die(void)
 {
-     if(pid > 0) my_kill(-pid);
+     if(pid > 0) 
+     {
+           my_kill(-pid);
+     }
      _exit(0);
 }
 
@@ -391,11 +401,11 @@ static void sigdie(int sig)
  *****************************************************************************/
 static void sig_to_fd(int sig)
 {
-    D("Signal %i routed to pipe\n", sig);
-    if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
-    {
-        D("Signal write to pipe failed\n");
-    }
+     D("Signal %i routed to pipe\n", sig);
+     if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
+     {
+          D("Signal write to pipe failed\n");
+     }
 }
 
 /******************************************************************************/
@@ -441,7 +451,9 @@ static void check_x_events(char * command)
 
           case Expose:
                if(ev.xexpose.count)
+               {
                     break;
+               }
           case ResizeRequest:
           case MapNotify:
                redraw();
@@ -479,35 +491,35 @@ static void check_x_events(char * command)
 static void normalise_window_coords(unsigned * width, unsigned * height,
                                     int * x, int * y)
 {
-    unsigned w = *width;
-    unsigned h = *height;
+     unsigned w = *width;
+     unsigned h = *height;
 
-    if(flags & H_FILL)        /* Dont bother if user wants to fill the window */
-    {
-        *x = *y = 0;
-        return;
-    }
+     if(flags & H_FILL)        /* Dont bother if user wants to fill the window */
+     {
+          *x = *y = 0;
+          return;
+     }
 
-    if((w > MAX_BUTTON_SIZE) & !(flags & H_MAXASPECT))
-    {
-        w = MAX_BUTTON_SIZE;
-    }
+     if((w > MAX_BUTTON_SIZE) & !(flags & H_MAXASPECT))
+     {
+          w = MAX_BUTTON_SIZE;
+     }
 
-    /* Keep the link button in a square window */
-    if(h < w)
-    {
-        w = h;
-    }
-    else if(h > w)
-    {
-        h = w;
-    }
+     /* Keep the link button in a square window */
+     if(h < w)
+     {
+          w = h;
+     }
+     else if(h > w)
+     {
+          h = w;
+     }
 
-    *x = (int) (*width - w)/2;
-    *y = (int) (*height - h)/2;
+     *x = (int) (*width - w)/2;
+     *y = (int) (*height - h)/2;
 
-    *height = h;
-    *width = w;
+     *height = h;
+     *width = w;
 }
 
 /******************************************************************************/
@@ -531,7 +543,9 @@ static void check_pipe_fd_events(void)
      if (n < 0)
      {
           if (errno == EINTR)
+          {
                return;
+          }
           exit(EX_UNAVAILABLE);
      }
   
@@ -644,11 +658,11 @@ int main(int argc, char **argv)
 
      if(argc > 3)
      {
-         nextHelper = argv[3];
+          nextHelper = argv[3];
      }
      else
      {
-         nextHelper = NULL;
+          nextHelper = NULL;
      }
 
      /* The application will use the $repeat variable */
@@ -794,8 +808,8 @@ int main(int argc, char **argv)
                }
                if(FD_ISSET(sig_fds[SIG_RD_FD], &fds))
                {
-                   int sig;
-                   read(sig_fds[SIG_RD_FD], &sig, sizeof(int));
+                    int sig;
+                    read(sig_fds[SIG_RD_FD], &sig, sizeof(int));
                }
           }
           D("SELECT OUT\n");
@@ -808,10 +822,13 @@ int main(int argc, char **argv)
                     if(pid > 0)
                     {
                          pid = -1;
-                         if(repeats > 0) my_start(command);
+                         if(repeats > 0) 
+                         {
+                              my_start(command);
+                         }
                     }
                }
           }
-    }
+     }
 }
 
