@@ -76,6 +76,8 @@ typedef struct data
 typedef struct mimetype
 {
      const char * type;
+
+     struct mimetype * pNext;
 } mimetype_t;
 
 typedef struct command
@@ -84,14 +86,16 @@ typedef struct command
      const char * cmd;
      const char * winname;
      const char * fmatchStr;
+
+     struct command * pNext;
 } command_t;
 
 typedef struct handle
 {
-     int num_types;
-     int num_cmds;
-     mimetype_t types[MAX_NUM_TYPES_PER_HANDLER];
-     command_t cmds[MAX_NUM_CMDS_PER_HANDLER];
+     mimetype_t * types;
+     command_t * cmds;
+
+     struct handle * pNext;
 } handler_t;
 
 typedef struct
@@ -102,19 +106,19 @@ typedef struct
 
 typedef struct 
 {
-    char name[SMALL_BUFFER_SIZE];
-    short exists;
+     char name[SMALL_BUFFER_SIZE];
+     short exists;
 } cacheEntry_t;
 
 /*****************************************************************************
  * Global variables
  *****************************************************************************/
+static const char *errMsg = NULL;
 static const char *config_fname;
 static const char *helper_fname;
 static const char *controller_fname;
 static const char *linker_fname;
-static int num_handlers;
-static handler_t handlers[MAX_NUM_HANDLERS];
+static handler_t * handlers = 0;
 static int browserApiMajorVer;
 static int browserApiMinorVer;
 
@@ -287,7 +291,9 @@ static void run(data_t * const THIS, const char *file, int fd)
 
      /* If no autostart seen and using controls dont autostart by default */
      if ((flags & (H_CONTROLS | H_LINKS)) && (THIS->autostartNotSeen))
+     {
           autostart = 0;
+     }
 
      snprintf(buffer, sizeof(buffer), "%d,%d,%d,%lu,%d,%d,%d,%d",
 	      flags,
@@ -366,6 +372,12 @@ static void run(data_t * const THIS, const char *file, int fd)
           launcher = helper_fname;
      }
 
+     if(launcher == 0)
+     {
+          D("No launcher defined");
+          _exit(EX_UNAVAILABLE);       /* Child exit, that's OK */
+     }
+
      D("Executing helper: %s %s %s %s %s %s\n",
        launcher,
        buffer,
@@ -376,16 +388,17 @@ static void run(data_t * const THIS, const char *file, int fd)
 
      if(nextHelper)
      {
-         execlp(launcher, launcher, buffer, THIS->command, nextHelper, NULL);
+          execlp(launcher, launcher, buffer, THIS->command, nextHelper, NULL);
      }
      else
      {
-         execlp(launcher, launcher, buffer, THIS->command, NULL);
+          execlp(launcher, launcher, buffer, THIS->command, NULL);
      }
 
      D("EXECLP FAILED!\n");
      
-     _exit(EX_UNAVAILABLE);
+     _exit(EX_UNAVAILABLE);       /* Child exit, that's OK */
+
 }
 
 /*****************************************************************************/
@@ -419,13 +432,12 @@ static int inpath(char *path, const char *file)
      for (i = 0; i < count; i++)
      {
 	  snprintf(buf, sizeof(buf), "%s/%s", path, file);
-	  D("stat(%s)\n", buf);
 	  if (stat(buf, &filestat) == 0)
 	  {
-	       D("yes\n");
+	       D("stat(%s) = yes\n", buf);
 	       return 1;
 	  }
-	  D("no\n");
+	  D("stat(%s) = no\n", buf);
 	  path += (strlen(path) + 1);
      }
 
@@ -458,12 +470,12 @@ static int find(const char *file)
 
      for(i = 0; i < cacheSize; i++)
      {
-         if( strcmp(cache[i].name, file) == 0)
-         {
-             const int exists = cache[i].exists;
-             D("find cache hit exists = %s\n", exists ? "yes" : "no");
-             return exists;
-         }
+          if( strcmp(cache[i].name, file) == 0)
+          {
+               const int exists = cache[i].exists;
+               D("find cache hit exists = %s\n", exists ? "yes" : "no");
+               return exists;
+          }
      }
 
      if (file[0] == '/')
@@ -508,29 +520,6 @@ static int find(const char *file)
      return exists;
 }
 
-/*****************************************************************************/
-/**
- * @brief Filter the mime handler
- *
- * Delete a mime handler if no command has been found for it.
- *
- *****************************************************************************/
-static void filter_previous_handler(void)
-{
-     handler_t *h;
-     if (num_handlers >= 1)
-     {
-	  h = &handlers[num_handlers-1];
-	  if (h->num_cmds == 0)
-	  {
-	       D("Empty handler: '%s'.\n", h->types[0].type);
-	       h->num_types = 0;
-	       num_handlers--;
-	  }
-     }
-}
-
-
 /*****************************************************************************/
 /**
  * @brief Allocate some memory
@@ -541,7 +530,7 @@ static void filter_previous_handler(void)
  *
  * @param[in] size The size of the memory to allocate
  *
- * @return Pointer to the memory allocated
+ * @return Pointer to the memory allocated or NULL
  *
  *****************************************************************************/
 static void * allocStaticMem(int size)
@@ -551,8 +540,12 @@ static void * allocStaticMem(int size)
 
      if(newIdx > MAX_STATIC_MEMORY_POOL)
      {
-	  fprintf(stderr, "MozPlugger: Too many entries in mozpluggerc\n");
-	  exit(1);
+          D("Out of static memory");
+
+	  errMsg = "MozPlugger: config file mozpluggerrc is too big - delete" \
+                                        "some handlers/commands or mimetypes";
+	  fprintf(stderr, "%s\n", errMsg);
+	  return NULL;
      }
 
      retVal = &staticPool[staticPoolIdx];
@@ -576,12 +569,19 @@ static void * allocStaticMem(int size)
  *****************************************************************************/
 static const char * makeStrStatic(const char * str, int len)
 {
-    /* plus one for string terminator */
-    char * const buf = allocStaticMem(len + 1);
+     /* plus one for string terminator */
+     char * const buf = allocStaticMem(len + 1);
 
-    strncpy(buf, str, len);
-    buf[len] ='\0';
-    return (const char *) buf;
+     if(buf)
+     {
+          strncpy(buf, str, len);
+          buf[len] ='\0';
+          return (const char *) buf;
+     }
+     else
+     {
+          return NULL;
+     }
 }
 
 
@@ -595,7 +595,7 @@ static const char * makeStrStatic(const char * str, int len)
  * @param[in] flag Pointer to flag being checked for
  * @param[out] c   Pointer to the found parameter
  *
- * @return Pointer to new position in text being parsed
+ * @return Pointer to new position in text being parsed or NULL if error
  *
  *****************************************************************************/
 static char *get_parameter(char *x, const char *flag, const char **c)
@@ -604,13 +604,17 @@ static char *get_parameter(char *x, const char *flag, const char **c)
      int len;
  
      /* skip spaces or tabs between flag name and parameter */
-     while ((*x == ' ' || *x == '\t')) 
-         x++;
+     while ((*x == ' ' || *x == '\t'))
+     {
+          x++;
+     }
 
      if (*x != '(')
      {
-	  fprintf(stderr, 
-                  "MozPlugger: Warning: Expected '(' after '%s'\n", flag);
+          D("Config error - expected '(' after '%s'\n", flag);
+
+	  errMsg = "MozPlugger: syntax error in mozpluggerrc config file";
+	  fprintf(stderr, "%s - expected '(' after '%s'\n", errMsg, flag);
 	  return NULL;
      }
      x++;
@@ -619,8 +623,17 @@ static char *get_parameter(char *x, const char *flag, const char **c)
      {
 	  len = end - x;
 	  *c = makeStrStatic(x, len);
+          /* No need to test if NULL as test done else where */
 	  x = end+1;
      }
+     else
+     {
+          D("Config error - expected ')'\n");
+
+	  errMsg = "MozPlugger: syntax error in mozpluggerrc config file";
+	  fprintf(stderr, "%s - expected ')' found nothing\n", errMsg);
+	  return NULL;
+     }
      return x;
 }
 
@@ -660,7 +673,8 @@ static int match_word(const char *line, const char *kw)
  *****************************************************************************/
 static int parse_flags(char **x, command_t *commandp)
 {
-     const static flag_t flags[] = {
+     const static flag_t flags[] = 
+     {
 	  { "repeat", 		H_REPEATCOUNT 	},
 	  { "loop", 		H_LOOP 		},
 	  { "stream", 		H_STREAM 	},
@@ -693,16 +707,26 @@ static int parse_flags(char **x, command_t *commandp)
                /* Get parameter for thoses commands with parameters */
                if(f->value & H_SWALLOW)
                {
-	            if ((*x = get_parameter(*x, f->name, &commandp->winname)))
-                        return 1;
+                    char * p = get_parameter(*x, f->name, &commandp->winname);
+	            if(p)
+                    {
+                         *x = p;
+                         return 1;
+                    }
                }
                else if(f->value & H_FMATCH)
                {
-	            if ((*x = get_parameter(*x, f->name, &commandp->fmatchStr)))
-                        return 1;
+	            char * p = get_parameter(*x, f->name, &commandp->fmatchStr);
+                    if(p)
+                    {
+                         *x = p;
+                         return 1;
+                    }
                }
                else
+               {
 	            return 1;
+               }
 	  }
      }
      return 0;
@@ -719,84 +743,153 @@ static int parse_flags(char **x, command_t *commandp)
  *****************************************************************************/
 static void read_config(FILE *f)
 {
-     handler_t *handler = NULL;
-     command_t *cmd = NULL;
-     mimetype_t *type = NULL;
+     int num_handlers = 0;
+
+     handler_t * prev_handler = NULL;
+     handler_t * handler = NULL;
+
+     command_t * cmd = NULL;
+     command_t * prev_cmd = NULL;
+
+     mimetype_t * type = NULL;
+     mimetype_t * prev_type = NULL;
+    
      char buffer[LARGE_BUFFER_SIZE];
      char file[SMALL_BUFFER_SIZE];
-     int have_commands = 1;
+     int seen_commands = 0;            /* Force the creation of first handler */
 
      D("read_config\n");
 
+     handler = (handler_t *) allocStaticMem(sizeof(handler_t));
+     if(handler == NULL)
+     {
+         return;
+     }
+     memset(handler, 0, sizeof(handler_t));
+
      while (fgets(buffer, sizeof(buffer), f))
      {
 	  D("::: %s", buffer);
 
 	  if (buffer[0] == '#' || buffer[0] == '\n' || buffer[0] == '\0')
+          {
 	       continue;
+          }
 
 	  if (buffer[strlen(buffer)-1] == '\n')
+          {
 	       buffer[strlen(buffer)-1] = 0;
+          }
 
 	  if (!isspace(buffer[0]))
 	  {
 	       /* Mime type */
 
-	       if (have_commands)
+	       if (seen_commands)
 	       {
+                    /* Store details of previous handler, if it has something
+                     * to offer */
+
+                    if(handler->cmds && handler->types)
+                    {
+                        /* Add at end of link list so this is the last */
+
+                        if(prev_handler)
+                        {
+                            /* Add to end of linked list of handlers */
+                            prev_handler->pNext = handler;
+                        }
+                        else
+                        {
+                            /* Add at header of link list of handlers */
+                            handlers = handler;
+                        }
+                        /* Remember the current end of linked list */
+                        prev_handler = handler;
+
+                        num_handlers++;
+                    }
+                    else
+                    {
+                        /* else - notice we dont free the static memory this is so
+                         * we always reserve same amount of memory irrespect of
+                         * what applications are currently installed */
+                        D("previous handler has no commands, so dropped\n");
+                    }
+
 		    D("-------------------------------------------\n");
-		    D("New handler\n");
+		    D("Starting new handler\n");
 		    D("-------------------------------------------\n");
 
-		    filter_previous_handler();
+		    handler = (handler_t *) allocStaticMem(sizeof(handler_t));
+                    if(handler == 0)
+                    {
+                        return;
+                    }
+                    memset(handler, 0, sizeof(handler_t));
 
-		    handler = &handlers[num_handlers++];
+                    prev_type = NULL;
+                    prev_cmd = NULL;
 
-		    if (!(num_handlers < MAX_NUM_HANDLERS))
-		    {
-			 fprintf(stderr, 
-                                 "MozPlugger: Error: Too many handlers (%d)\n", 
-                                 num_handlers);
-			 exit(1);
-		    }
-
-		    have_commands = 0;
+		    seen_commands = 0;
 	       }
 
 	       D("New mime type\n");
 
-	       type = &handler->types[handler->num_types++];
+               type = (mimetype_t *) allocStaticMem(sizeof(mimetype_t));
+               if(type == 0)
+               {
+                    continue;
+               }
+               memset(type, 0, sizeof(mimetype_t));
 
-	       if (!(handler->num_types < MAX_NUM_TYPES_PER_HANDLER))
-	       {
-		    fprintf(stderr, 
-                            "MozPlugger: Error: Too many types (%d) for handler"
-                            " %d (%s)\n",
-			    handler->num_types, num_handlers, 
-                            handler->types[0].type);
-		    exit(1);
-	       }
 	       type->type = makeStrStatic(buffer, strlen(buffer));
+               if(type->type == 0)
+               {
+                    continue;
+               }
+
+               if(prev_type)
+               {
+                    /* Add to end of linked list of handlers */
+                    prev_type->pNext = type;
+               }
+               else
+               {
+                    /* Add at header of link list of handlers */
+                    handler->types = type;
+               }
+               /* Remember the current end of linked list */
+               prev_type = type;
 	  }
 	  else
 	  {
 	       /* Command */
 
 	       char *x = buffer + 1;
-	       while (isspace(*x)) x++;
+	       while (isspace(*x))
+               {
+                    x++;
+               }
+
 	       if (!*x)
 	       {
 		    D("Empty command.\n");
-		    have_commands++;
+		    seen_commands = 1;
 		    continue;
 	       }
 
 	       D("New command\n");
 
-	       have_commands++;
+	       seen_commands = 1;
 
-	       cmd = &handler->cmds[handler->num_cmds];
+               cmd = (command_t *) allocStaticMem(sizeof(command_t));
+               if(cmd == NULL)
+               {
+                    continue;
+               }
 	       memset(cmd, 0, sizeof(command_t));
+               
 
 	       D("Parsing %s\n", x);
 
@@ -808,46 +901,78 @@ static void read_config(FILE *f)
 		    }
 		    else if (!parse_flags(&x, cmd))
 		    {
-			 fprintf(stderr, 
-                                 "MozPlugger: Warning: Unknown directive: %s\n",
-                                 x);
-			 x += strlen(x);
+                         D("Config error - unknown directive %s\n", x);
+
+	                 errMsg = "MozPlugger: syntax error in mozpluggerrc config file";
+		         fprintf(stderr, "%s - unknown directive: %s\n", errMsg,  x);
+		         x += strlen(x);
 		    }
 	       }
 	       if (*x == ':')
 	       {
-		    x++;
-		    while (isspace(*x)) x++;
+	            x++;
+		    while (isspace(*x))
+                    {
+                         x++;
+                    }
 
 		    if (sscanf(x, "if %"SMALL_BUFFER_SIZE_STR"s", file) != 1
-			&& sscanf(x, "%"SMALL_BUFFER_SIZE_STR"s", file) != 1)
-			 continue;
+			     && sscanf(x, "%"SMALL_BUFFER_SIZE_STR"s", file) != 1)
+                    {
+		         continue;
+                    }
 
 		    if (!find(file))
-			 continue;
+                    {
+		         continue;
+                    }
 
 		    cmd->cmd = makeStrStatic(x, strlen(x));
+                    if(cmd->cmd == 0)
+                    {
+                         continue;
+                    }
 	       }
 	       else
 	       {
-		    D("No column? (%s)\n", x);
+	            D("No column? (%s)\n", x);
+                    continue;
 	       }
 
-	       handler->num_cmds++;
-
-	       if (!(handler->num_cmds < MAX_NUM_CMDS_PER_HANDLER))
-	       {
-		    fprintf(stderr, 
-                            "MozPlugger: Error: Too many commands (%d) for"
-                            " handler %d (%s)\n",
-			    handler->num_cmds, num_handlers, 
-                            handler->types[0].type);
-		    exit(1);
-	       }
-	  }
+               if(prev_cmd)
+               {
+                    /* Add to end of linked list of handlers */
+                    prev_cmd->pNext = cmd;
+               }
+               else
+               {
+                    /* Add at header of link list of handlers */
+                    handler->cmds = cmd;
+               }
+               /* Remember the current end of linked list */
+               prev_cmd = cmd;
+          }
      }
 
-     filter_previous_handler();
+     if(handler->cmds && handler->types)
+     {
+          if(prev_handler)
+          {
+               prev_handler->pNext = handler;
+          }
+          else
+          {
+               handlers = handler;
+          }
+          num_handlers++;
+     }
+     else
+     {
+          /* else - notice we dont free the static memory this is so
+           * we always reserve same amount of memory irrespect of
+           * what applications are currently installed */
+          D("previous handler has no commands, so dropped\n");
+     }
 
      D("Num handlers: %d\n", num_handlers);
 }
@@ -873,41 +998,71 @@ static int find_helper_file(const char *basename, int (*cb)(const char *))
      if ((tmp = getenv("HOME")))
      {
 	  snprintf(fname, sizeof(fname), "%s/.netscape/%s", tmp, basename);
-	  if (cb(fname)) return 1;
+	  if (cb(fname)) 
+          {
+               return 1;
+          }
 
 	  snprintf(fname, sizeof(fname), "%s/.mozilla/%s", tmp, basename);
-	  if (cb(fname)) return 1;
+	  if (cb(fname)) 
+          {
+               return 1;
+          }
 
 	  snprintf(fname, sizeof(fname), "%s/.opera/%s", tmp, basename);
-	  if (cb(fname)) return 1;
+	  if (cb(fname)) 
+          {
+               return 1;
+          }
      }
 
      if ((tmp = getenv("MOZILLA_HOME")))
      {
 	  snprintf(fname, sizeof(fname), "%s/%s", tmp, basename);
-	  if (cb(fname)) return 1;
+	  if (cb(fname)) 
+          {
+               return 1;
+          }
      }
 
      if ((tmp = getenv("OPERA_DIR")))
      {
 	  snprintf(fname, sizeof(fname), "%s/%s", tmp, basename);
-	  if (cb(fname)) return 1;
+	  if (cb(fname)) 
+          {
+               return 1;
+          }
      }
 
      snprintf(fname, sizeof(fname), "/etc/%s", basename);
-     if (cb(fname)) return 1;
+     if (cb(fname)) 
+     {
+          return 1;
+     }
 
      snprintf(fname, sizeof(fname), "/usr/etc/%s", basename);
-     if (cb(fname)) return 1;
+     if (cb(fname)) 
+     {
+          return 1;
+     }
 
      snprintf(fname, sizeof(fname), "/usr/local/mozilla/%s", basename);
-     if (cb(fname)) return 1;
+     if (cb(fname)) 
+     {
+          return 1;
+     }
 
      snprintf(fname, sizeof(fname), "/usr/local/netscape/%s", basename);
-     if (cb(fname)) return 1;
+     if (cb(fname)) 
+     {
+          return 1;
+     }
 
-     if (cb(basename)) return 1;
-  
+     if (cb(basename)) 
+     {
+          return 1;
+     }
+ 
      return 0;
 }
 
@@ -933,16 +1088,25 @@ static int read_config_cb(const char *fname)
      D("READ_CONFIG(%s)\n", fname);
 
      fd = open(fname, O_RDONLY);
-     if (fd < 0) return 0;
+     if (fd < 0)
+     {
+          /* Not an error! */
+          D("could not open '%s'\n", fname);
+          return 0;
+     }
 
      if (pipe(m4out) < 0)
      {
+          D("Failed to create pipe\n");
 	  perror("pipe");
 	  return 0;
      }
 
      if ((pid = fork()) == -1)
+     {
+          D("Failed to fork\n");
 	  return 0;
+     }
 
      if (!pid)
      {
@@ -956,7 +1120,8 @@ static int read_config_cb(const char *fname)
 
 	  execlp("m4", "m4", NULL);
 	  fprintf(stderr, "MozPlugger: Error: Failed to execute m4.\n");
-	  exit(1);
+	  exit(1);        /* Child exit, that's OK */
+
      }
      else
      {
@@ -964,13 +1129,26 @@ static int read_config_cb(const char *fname)
 	  close(fd);
 	  
 	  fp = fdopen(m4out[0], "r");
-	  if (!fp) return 0;
+	  if (!fp)
+          {
+               D("Failed to open pipe\n");
+               return 0;
+          }
 	  read_config(fp);
 	  fclose(fp);
 	  
-	  waitpid(pid, 0, 0);
-	  config_fname = makeStrStatic(fname, strlen(fname));
+          int status;
+	  waitpid(pid, &status, 0);
+          D("M4 exit status was %i\n", WEXITSTATUS(status));
 
+          if(WEXITSTATUS(status) != 0)
+          {
+               errMsg = "Mozplugger: M4 parsing of config generated error";
+               fprintf(stderr, "%s\n",errMsg);
+               D("%s\n", errMsg);
+          }
+	  config_fname = makeStrStatic(fname, strlen(fname));
+          /* No need to test config_fname is NULL, done elsewhere */
      }
      return 1;
 }
@@ -995,6 +1173,7 @@ static int find_plugger_helper_cb(const char *fname)
           return 0;
      }
      helper_fname = makeStrStatic(fname, strlen(fname));
+     /* No need to test helper_fname is NULL, done elsewhere */
      return 1;
 }
 
@@ -1003,7 +1182,7 @@ static int find_plugger_helper_cb(const char *fname)
  * @brief Find mozplugger helper app callback function
  *
  * Check if the helper executable exists (using stat), if so save the filename
- * to the global 'helper_fname' variable
+ * to the global 'controller_fname' variable
  *
  * @param[in] fname Full application name (including path)
  *
@@ -1018,6 +1197,7 @@ static int find_plugger_controller_cb(const char *fname)
           return 0;
      }
      controller_fname = makeStrStatic(fname, strlen(fname));
+     /* No need to test controller_fname is NULL, done elsewhere */
      return 1;
 }
 
@@ -1026,7 +1206,7 @@ static int find_plugger_controller_cb(const char *fname)
  * @brief Find mozplugger helper app callback function
  *
  * Check if the helper executable exists (using stat), if so save the filename
- * to the global 'helper_fname' variable
+ * to the global 'linker_fname' variable
  *
  * @param[in] fname Full application name (including path)
  *
@@ -1041,6 +1221,7 @@ static int find_plugger_linker_cb(const char *fname)
           return 0;
      }
      linker_fname = makeStrStatic(fname, strlen(fname));
+     /* No need to test linker_fname is NULL, done elsewhere */
      return 1;
 }
 
@@ -1056,14 +1237,18 @@ static int find_plugger_linker_cb(const char *fname)
  *****************************************************************************/
 static void do_read_config(void)
 {
-     if (num_handlers > 0) return;
+     if (handlers)
+     {
+          return;
+     }
 
      D("do_read_config\n");
   
      if (!find_helper_file("mozpluggerrc", read_config_cb))
      {
-	  fprintf(stderr, "MozPlugger: Warning: Unable to find the"
-                          " mozpluggerrc file.\n");
+          errMsg = "Mozplugger: Installation error - failed to locate mozpluggerrc";
+          fprintf(stderr, "%s\n",errMsg);
+          D("%s\n", errMsg);
 	  return;
      }
 
@@ -1075,8 +1260,9 @@ static void do_read_config(void)
 	  } 
           else 
           { 
-	       fprintf(stderr, "MozPlugger: Warning: Unable to find"
-                       " mozplugger-helper.\n");
+               errMsg = "Mozplugger: Installation error - failed to locate mozplugger-helper";
+               fprintf(stderr, "%s\n",errMsg);
+               D("%s\n", errMsg);
 	       return;
 	  }
      }
@@ -1089,8 +1275,9 @@ static void do_read_config(void)
 	  } 
           else 
           {
-	       fprintf(stderr, "MozPlugger: Warning: Unable to find"
-                               " mozplugger-controller.\n");
+               errMsg = "Mozplugger: Installation error - failed to locate mozplugger-controller";
+               fprintf(stderr, "%s\n",errMsg);
+               D("%s\n", errMsg);
 	       return;
 	  }
      }
@@ -1103,8 +1290,9 @@ static void do_read_config(void)
 	  } 
           else 
           {
-	       fprintf(stderr, "MozPlugger: Warning: Unable to find"
-                               " mozplugger-linker.\n");
+               errMsg = "Mozplugger: Installation error - failed to locate mozplugger-linker";
+               fprintf(stderr, "%s\n",errMsg);
+               D("%s\n", errMsg);
 	       return;
 	  }
      }
@@ -1339,19 +1527,18 @@ static int match_handler(handler_t *h, data_t * const THIS,
 {
      mimetype_t *m;
      command_t *c;
-     int mid, cid;
 
      D("-------------------------------------------\n");
      D("Commands for this handle at (%p):\n", h->cmds);
 
-     for (mid = 0; mid < h->num_types; mid++)
+     m = h->types;
+     while(m)
      {
-	  m = &h->types[mid];
 	  if (match_mime_type(THIS, m))
 	  {
-	       for (cid = 0; cid < h->num_cmds; cid++)
+               c = h->cmds;
+               while(c)
 	       {
-		    c = &h->cmds[cid];
 		    if (match_command(THIS, streamOnly, c, url))
 		    {
 			 THIS->cmd_flags = c->flags;
@@ -1359,8 +1546,10 @@ static int match_handler(handler_t *h, data_t * const THIS,
 			 THIS->winname = c->winname;
 			 return 1;
 		    }
+                    c = c->pNext;
 	       }
 	  }
+          m = m->pNext;
      }
      return 0;
 }
@@ -1380,19 +1569,21 @@ static int match_handler(handler_t *h, data_t * const THIS,
  *****************************************************************************/
 static int find_command(data_t * const THIS, int streamOnly, const char * url)
 {
-     int hid;
+     handler_t * h;
 
      D("find_command...\n");
 
      do_read_config();
 
-     for (hid = 0; hid < num_handlers; hid++)
+     h = handlers;
+     while(h)
      {
-	  if (match_handler(&handlers[hid], THIS, streamOnly, url))
+	  if (match_handler(h, THIS, streamOnly, url))
 	  {
 	       D("Command found.\n");
 	       return 1;
 	  }
+          h = h->pNext;
      }
 
      D("No command found.\n");
@@ -1414,45 +1605,60 @@ char *NPP_GetMIMEDescription(void)
      handler_t *h;
      mimetype_t *m;
      int size_required;
-     int hid, mid;
 
      D("GetMIMEDescription\n");
 
      do_read_config();
 
+#ifdef DEBUG
+     {
+          const int free = MAX_STATIC_MEMORY_POOL - staticPoolIdx;
+          D("Static Pool used=%i, free=%i\n", staticPoolIdx, free);
+     }
+#endif
+
      size_required = 0;
 
-     for (hid = 0; hid < num_handlers; hid++)
+     h = handlers;
+     while(h)
      {
-	  h = &handlers[hid];
-	  for (mid = 0; mid < h->num_types; mid++)
+          m = h->types;
+          while(m)
 	  {
-	       m = &h->types[mid];
 	       size_required += strlen(m->type)+1;
+               m = m->pNext;
 	  }
+          h = h->pNext;
      }
 
      D("Size required=%d\n", size_required);
 
      if (!(x = (char *)malloc(size_required+1)))
+     {
 	  return 0;
+     }
 
      D("Malloc did not fail\n");
 
      y = x;
 
-     for (hid = 0; hid < num_handlers; hid++)
+     h = handlers;
+     while(h)
      {
-	  h = &handlers[hid];
-	  for (mid = 0; mid < h->num_types; mid++)
+          m = h->types;
+          while(m)
 	  {
-	       m = &h->types[mid];
 	       memcpy(y,m->type,strlen(m->type));
 	       y+=strlen(m->type);
 	       *(y++)=';';
+               m = m->pNext;
 	  }
+          h = h->pNext;
+     }
+     if (x != y) 
+     {
+          y--;
      }
-     if (x != y) y--;
      *(y++) = 0;
 
      D("Getmimedescription done: %s\n", x);
@@ -1511,7 +1717,7 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 		   " <tr><td>Debug file:</td><td>%s/" DEBUG_FILENAME 
                    "</td></tr> "
 #endif
-		   " </table> "
+		   " </table> %s"
 		   "<br clear=all>",
 		   config_fname ? config_fname : "Not found!",
 		   helper_fname ? helper_fname : "Not found!",
@@ -1520,13 +1726,15 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 #ifdef DEBUG
                    , get_debug_path()
 #endif
+                   , errMsg ? errMsg : ""
                   );
+          errMsg = NULL;
 	  *((char **)value) = desc_buffer;
 	  break;
 
      case NPPVpluginNeedsXEmbed:
-          D("GET Plugin 'needs XEmbed' not implemented\n"); 
-	  err = NPERR_GENERIC_ERROR;
+          D("GET Plugin 'needs XEmbed', returned false\n"); 
+          *((int *)value) = 0;
           break; 
 
      default:
@@ -1537,6 +1745,33 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
      return err;
 }
 
+/*****************************************************************************/
+/** NPP Set value
+ *
+ * Let Mozilla set things on mozplugger. 
+ *
+ * @param[in] instance Pointer to plugin instance data
+ * @param[in] variable Name of variable to get (enum)
+ * @param[in] value    The value to set
+ *
+ * @return Returns error code if problem
+ *
+ *****************************************************************************/
+NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value)
+{
+     NPError err = NPERR_NO_ERROR;
+  
+     switch (variable)
+     {
+     default:
+          D("SET Plugin value %d not implemented\n", variable);
+	  err = NPERR_GENERIC_ERROR;
+          break;
+     }
+     return err;
+}
+
+
 /*****************************************************************************/
 /**
  * @brief Special version of atoi
@@ -1586,8 +1821,8 @@ static int my_atoi(const char *s, int my_true, int my_false)
  *****************************************************************************/
 NPError NPP_New(NPMIMEType pluginType,
 		NPP instance,
-		uint16 mode,
-		int16 argc,
+		uint16_t mode,
+		int16_t argc,
 		char* argn[],
 		char* argv[],
 		NPSavedData* saved)
@@ -1635,21 +1870,25 @@ NPError NPP_New(NPMIMEType pluginType,
      THIS->autostartNotSeen = 1;
      if(mode == NP_EMBED)
      {
-        THIS->mode_flags = H_EMBED;
+         THIS->mode_flags = H_EMBED;
      }
      else
      {
-        THIS->mode_flags = H_NOEMBED;
+         THIS->mode_flags = H_NOEMBED;
      }
 
      if (!(THIS->mimetype = strdup(pluginType)))
+     {
 	  return NPERR_OUT_OF_MEMORY_ERROR;
+     }
 
      THIS->num_arguments = argc;
      if (!(THIS->args = (argument_t *)NPN_MemAlloc(
-                                          (uint32)(sizeof(argument_t) * argc))))
+                                          (uint32_t)(sizeof(argument_t) * argc))))
+     {
 	  return NPERR_OUT_OF_MEMORY_ERROR;
-     
+     }
+
      for (e = 0; e < argc; e++)
      {
 	  if (strcasecmp("loop", argn[e]) == 0)
@@ -1710,7 +1949,9 @@ NPError NPP_New(NPMIMEType pluginType,
                const int len = strlen(argn[e]) + 5;
 
     	       if (!(THIS->args[e].name = (char *)malloc(len)))
-	             return NPERR_OUT_OF_MEMORY_ERROR;
+               {
+	            return NPERR_OUT_OF_MEMORY_ERROR;
+               }
 	       snprintf(THIS->args[e].name, len, "VAR_%s", argn[e]);
  	       THIS->args[e].value = argv[e] ? strdup(argv[e]) : NULL;
           }
@@ -1735,7 +1976,6 @@ NPError NPP_New(NPMIMEType pluginType,
                    THIS->mode_flags &= ~(H_EMBED | H_NOEMBED);
                    THIS->mode_flags |= H_LINKS;
                }
-
           }
      }
      else if (data_idx >= 0)
@@ -1768,13 +2008,13 @@ NPError NPP_New(NPMIMEType pluginType,
              || (strncmp(url, "rtspu://", 8) == 0)   /* RTSP over UDP */
              || (strncmp(url, "rtspt://", 8) == 0))  /* RTSP over TCP */
 	  {
-	        D("Detected MMS -> url=%s\n", url);
+	       D("Detected MMS -> url=%s\n", url);
 
-                if( !safeURL(url))
-                {
+               if( !safeURL(url))
+               {
 	            return NPERR_GENERIC_ERROR;
-                }
-	        THIS->mms = url;
+               }
+	       THIS->mms = url;
 	  }
           else 
           {
@@ -1794,6 +2034,7 @@ NPError NPP_New(NPMIMEType pluginType,
 
 	                 fprintf(stderr, "MozPlugger: Warning: Couldn't get"
                                  "%s\n", url);
+                         return NPERR_GENERIC_ERROR;
                     }
                }
           }
@@ -1833,11 +2074,13 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
      {
 	  /* Kill the mozplugger-helper process and his sons */
  	  if (THIS->pid > 0)
+          {
 	       my_kill(-THIS->pid);
-
+          }
 	  if (THIS->fd > 0)
+          {
 	       close(THIS->fd);
-
+          }
 	  for (e = 0; e < THIS->num_arguments; e++)
 	  {
 	       free((char *)THIS->args[e].name);
@@ -1887,11 +2130,15 @@ static void new_child(NPP instance, const char* fname)
      THIS = instance->pdata;
 
      if (THIS->pid != -1)
+     {
 	  return;
+     }
 
      /* Guard against spawning helper if no command! */
      if(THIS->command == 0)
-         return;
+     {
+          return;
+     }
 
      if (socketpair(AF_UNIX, SOCK_STREAM, 0, pipe) < 0)
      {
@@ -1912,7 +2159,7 @@ static void new_child(NPP instance, const char* fname)
 	  D("CHILD RUNNING run() [2]\n");
 	  run(THIS, fname, pipe[1]);
 
-	  _exit(EX_UNAVAILABLE);
+	  _exit(EX_UNAVAILABLE);       /* Child exit, that's OK */
      }
      else
      {
@@ -1941,7 +2188,7 @@ NPError NPP_NewStream(NPP instance,
 		      NPMIMEType type,
 		      NPStream *stream, 
 		      NPBool seekable,
-		      uint16 *stype)
+		      uint16_t *stype)
 {
      char * savedMimetype = NULL;
      data_t * THIS;
@@ -1965,7 +2212,9 @@ NPError NPP_NewStream(NPP instance,
       * mozpluggerc instead... */
      if (!strncasecmp("image/", type, 6) ||
 	 !strncasecmp("x-image/", type, 6))
+     {
 	  THIS->repeats = 1;
+     }
 
      /*  Replace the stream's URL with the URL in THIS->href if it
       *  exists.  Since we're going to be replacing <embed>'s src with
@@ -1992,10 +2241,10 @@ NPError NPP_NewStream(NPP instance,
       * the original tag in the mime type. */
      if(strcmp(type, THIS->mimetype) != 0)
      {
-         D("Mismatching mimetype reported, originally was \'%s\' now '\%s' "
-           "for url %s\n", THIS->mimetype, type, stream->url);
-         savedMimetype = THIS->mimetype;
-         THIS->mimetype = strdup(type);
+          D("Mismatching mimetype reported, originally was \'%s\' now '\%s' "
+            "for url %s\n", THIS->mimetype, type, stream->url);
+          savedMimetype = THIS->mimetype;
+          THIS->mimetype = strdup(type);
      }
 
      /* Find a suitable command, fall back to savedMimeType if defined */
@@ -2003,15 +2252,23 @@ NPError NPP_NewStream(NPP instance,
      {
           if(savedMimetype)       /* Lets try the saved one */
           {
-              free(THIS->mimetype);
-              THIS->mimetype = savedMimetype;
-              savedMimetype = NULL;
+               free(THIS->mimetype);
+               THIS->mimetype = savedMimetype;
+               savedMimetype = NULL;
           }
           else 
           {
-	      NPN_Status(instance, 
+               if(errMsg)
+               {
+                    NPN_Status(instance, errMsg);
+                    errMsg = NULL;
+               }
+               else
+               {
+	            NPN_Status(instance, 
                          "MozPlugger: No appropriate application found.");
-	      return NPERR_GENERIC_ERROR;
+               }
+	       return NPERR_GENERIC_ERROR;
           }
      }
      free(savedMimetype);     /* discard the saved Mime type */
@@ -2075,7 +2332,7 @@ static void resize_window(NPP instance)
      /* Mozilla work around not require for versions > 0.13 */
      if((browserApiMajorVer > 0) || (browserApiMinorVer > 13))
      {
-         return;
+          return;
      }
      THIS = instance->pdata;
 
@@ -2114,10 +2371,14 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
      D("SetWindow() - instance=%p\n", instance);
 
      if (!instance)
+     {
 	  return NPERR_INVALID_INSTANCE_ERROR;
+     }
 
      if (!window)
+     {
 	  return NPERR_NO_ERROR;
+     }
  
      /* TODO - Should really pass to helper / controller the fact that the
       * window has disappeared! - need to check the consequences first
@@ -2129,8 +2390,10 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
      }
 
      if (!window->ws_info)
+     {
 	  return NPERR_NO_ERROR;
-    
+     }
+
      THIS = instance->pdata;
 
      THIS->display = ((NPSetWindowCallbackStruct *)window->ws_info)->display;
@@ -2146,8 +2409,16 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
                if (!find_command(THIS, 1, THIS->mms))
                {
 	           THIS->mms = NULL;
-	           NPN_Status(instance, 
+                   if(errMsg)
+                   {
+                        NPN_Status(instance, errMsg);
+                        errMsg = NULL;
+                   }
+                   else
+                   {
+	                NPN_Status(instance, 
                           "MozPlugger: No appropriate application found.");
+                   }
  	           return NPERR_GENERIC_ERROR;
                }
           }
@@ -2221,8 +2492,8 @@ NPError NPP_Initialize(void)
 
 #ifdef DEBUG
      {
-         const int free = MAX_STATIC_MEMORY_POOL - staticPoolIdx;
-         D("Static Pool used=%i, free=%i\n", staticPoolIdx, free);
+          const int free = MAX_STATIC_MEMORY_POOL - staticPoolIdx;
+          D("Static Pool used=%i, free=%i\n", staticPoolIdx, free);
      }
 #endif
 
@@ -2279,10 +2550,10 @@ void NPP_Print(NPP instance, NPPrint* printInfo)
  * @return Always returns value of passed in 'len'
  *
  *****************************************************************************/
-int32 NPP_Write(NPP instance,
+int32_t NPP_Write(NPP instance,
 		NPStream *stream,
-		int32 offset,
-		int32 len,
+		int32_t offset,
+		int32_t len,
 		void *buf)
 {
      D("Write(%d) - instance=%p\n", (int) len, instance);
@@ -2305,17 +2576,17 @@ int32 NPP_Write(NPP instance,
  * @return Always returns zero
  *
  *****************************************************************************/
-int32 NPP_WriteReady(NPP instance, NPStream *stream)
+int32_t NPP_WriteReady(NPP instance, NPStream *stream)
 {
-    D("WriteReady() - instance=%p\n", instance);
-    if (instance)
-    {
-	 /* Tell the browser that it can finish with the stream
+     D("WriteReady() - instance=%p\n", instance);
+     if (instance)
+     {
+	  /* Tell the browser that it can finish with the stream
 	    (actually we just wanted the name of the stream!)
 	    And not the stream data. */
-        NPN_DestroyStream(instance, stream, NPRES_DONE);
-    }
-    return 0;
+          NPN_DestroyStream(instance, stream, NPRES_DONE);
+     }
+     return 0;
 }
 
 /*****************************************************************************/
@@ -2333,5 +2604,5 @@ int32 NPP_WriteReady(NPP instance, NPStream *stream)
  *****************************************************************************/
 void NPP_URLNotify(NPP instance, const char * url, NPReason reason, void * notifyData)
 {
-    D("URLNotify() - instance=%p url=%s reason=%i\n", instance, url, reason);
+     D("URLNotify() - instance=%p url=%s reason=%i\n", instance, url, reason);
 }
