@@ -44,32 +44,18 @@
  * - Wrapper function to interface with the Netscape Navigator
  *
  * dp Suresh <dp@netscape.com>
+ * Peter Leese - modified for mozplugger
  *
- *----------------------------------------------------------------------
- * PLUGIN DEVELOPERS:
- *  YOU WILL NOT NEED TO EDIT THIS FILE.
- *----------------------------------------------------------------------
  */
 
 #define XP_UNIX 1
 
 #include <stdio.h>
+#include <string.h>
+
 #include "npapi.h"
 #include "npupp.h"
 
-/*
- * Define PLUGIN_TRACE to have the wrapper functions print
- * messages to stderr whenever they are called.
- */
-
-#ifdef PLUGIN_TRACE
-#include <stdio.h>
-#define PLUGINDEBUGSTR(msg) fprintf(stderr, "%s\n", msg)
-#else
-#define PLUGINDEBUGSTR(msg)
-#endif
-
-
 /***********************************************************************
  *
  * Globals
@@ -102,369 +88,485 @@ NPN_Version(int* plugin_major, int* plugin_minor,
     *netscape_minor = gNetscapeFuncs.version & 0xFF;
 }
 
+/******************************************************************************/
 NPError
 NPN_GetValue(NPP instance, NPNVariable variable, void *r_value)
 {
-    return CallNPN_GetValueProc(gNetscapeFuncs.getvalue,
-                    instance, variable, r_value);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_GetValueUPP func = gNetscapeFuncs.getvalue;
+    if(func)
+    {
+        retVal = (*func)(instance, variable, r_value);
+    }
+    return retVal;
 }
 
-#if 0 /* Peter Leese - new stuff commented out for now */
-
+/******************************************************************************/
 NPError
 NPN_SetValue(NPP instance, NPPVariable variable, void *value)
 {
-    return CallNPN_SetValueProc(gNetscapeFuncs.setvalue,
-                    instance, variable, value);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_SetValueUPP func = gNetscapeFuncs.setvalue;
+    if(func)
+    {
+        retVal = (*func)(instance, variable, value);
+    }
+    return retVal;
 }
 
-#endif
-
+/******************************************************************************/
 NPError
 NPN_GetURL(NPP instance, const char* url, const char* window)
 {
-    return CallNPN_GetURLProc(gNetscapeFuncs.geturl, instance, url, window);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_GetURLUPP func = gNetscapeFuncs.geturl;
+    if(func)
+    {
+        retVal = (*func)(instance, url, window);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPError
 NPN_GetURLNotify(NPP instance, const char* url, const char* window, void* notifyData)
 {
-    return CallNPN_GetURLNotifyProc(gNetscapeFuncs.geturlnotify, instance, url, window, notifyData);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_GetURLNotifyUPP func = gNetscapeFuncs.geturlnotify;
+    if(func)
+    {
+        retVal = (*func)(instance, url, window, notifyData);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPError
 NPN_PostURL(NPP instance, const char* url, const char* window,
-         uint32 len, const char* buf, NPBool file)
+         uint32_t len, const char* buf, NPBool file)
 {
-    return CallNPN_PostURLProc(gNetscapeFuncs.posturl, instance,
-                    url, window, len, buf, file);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_PostURLUPP func = gNetscapeFuncs.posturl;
+    if(func)
+    {
+        retVal = (*func)(instance, url, window, len, buf, file);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPError
-NPN_PostURLNotify(NPP instance, const char* url, const char* window, uint32 len,
+NPN_PostURLNotify(NPP instance, const char* url, const char* window, uint32_t len,
                   const char* buf, NPBool file, void* notifyData)
 {
-    return CallNPN_PostURLNotifyProc(gNetscapeFuncs.posturlnotify,
-            instance, url, window, len, buf, file, notifyData);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_PostURLNotifyUPP func = gNetscapeFuncs.posturlnotify;
+    if(func)
+    {
+        retVal = (*func)(instance, url, window, len, buf, file, notifyData);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPError
 NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
 {
-    return CallNPN_RequestReadProc(gNetscapeFuncs.requestread,
-                    stream, rangeList);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_RequestReadUPP func = gNetscapeFuncs.requestread;
+    if(func)
+    {
+        retVal = (*func)(stream, rangeList);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPError
 NPN_NewStream(NPP instance, NPMIMEType type, const char *window,
           NPStream** stream_ptr)
 {
-    return CallNPN_NewStreamProc(gNetscapeFuncs.newstream, instance,
-                    type, window, stream_ptr);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_NewStreamUPP func = gNetscapeFuncs.newstream;
+    if(func)
+    {
+        retVal = (*func)(instance,  type, window, stream_ptr);
+    }
+    return retVal;
 }
 
-int32
-NPN_Write(NPP instance, NPStream* stream, int32 len, void* buffer)
+/******************************************************************************/
+int32_t
+NPN_Write(NPP instance, NPStream* stream, int32_t len, void* buffer)
 {
-    return CallNPN_WriteProc(gNetscapeFuncs.write, instance,
-                    stream, len, buffer);
+    int32_t retVal = 0;
+    NPN_WriteUPP func = gNetscapeFuncs.write;
+    if(func)
+    {
+        retVal = (*func)(instance, stream, len, buffer);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPError
 NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason)
 {
-    return CallNPN_DestroyStreamProc(gNetscapeFuncs.destroystream,
-                        instance, stream, reason);
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    NPN_DestroyStreamUPP func = gNetscapeFuncs.destroystream;
+    if(func)
+    {
+        retVal = (*func)(instance, stream, reason);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 void
 NPN_Status(NPP instance, const char* message)
 {
-    CallNPN_StatusProc(gNetscapeFuncs.status, instance, message);
+    NPN_StatusUPP func = gNetscapeFuncs.status;
+    if(func)
+    {
+        (*func)(instance, message);
+    }
 }
 
+/******************************************************************************/
 const char*
 NPN_UserAgent(NPP instance)
 {
-    return CallNPN_UserAgentProc(gNetscapeFuncs.uagent, instance);
+    const char * retVal = 0;
+    NPN_UserAgentUPP func = gNetscapeFuncs.uagent;
+    if(func)
+    {
+        retVal = (*func)(instance);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 void*
-NPN_MemAlloc(uint32 size)
+NPN_MemAlloc(uint32_t size)
 {
-    return CallNPN_MemAllocProc(gNetscapeFuncs.memalloc, size);
+    void * retVal = 0;
+    NPN_MemAllocUPP func = gNetscapeFuncs.memalloc;
+    if(func)
+    {
+        retVal = (*func)(size);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 void NPN_MemFree(void* ptr)
 {
-    CallNPN_MemFreeProc(gNetscapeFuncs.memfree, ptr);
+    NPN_MemFreeUPP func = gNetscapeFuncs.memfree;
+    if(func)
+    {
+        (*func)(ptr);
+    }
 }
 
-uint32 NPN_MemFlush(uint32 size)
+/******************************************************************************/
+uint32_t NPN_MemFlush(uint32_t size)
 {
-    return CallNPN_MemFlushProc(gNetscapeFuncs.memflush, size);
+    uint32_t retVal = 0;
+    NPN_MemFlushUPP func = gNetscapeFuncs.memflush;
+    if(func)
+    {
+        retVal = (*func)(size);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 void NPN_ReloadPlugins(NPBool reloadPages)
 {
-    CallNPN_ReloadPluginsProc(gNetscapeFuncs.reloadplugins, reloadPages);
+    NPN_ReloadPluginsUPP func = gNetscapeFuncs.reloadplugins;
+    if(func)
+    {
+        (*func)(reloadPages);
+    }
 }
 
-#if 0 /* Peter Leese - new stuff commented out for now */
-
+/******************************************************************************/
 void
 NPN_InvalidateRect(NPP instance, NPRect *invalidRect)
 {
-    CallNPN_InvalidateRectProc(gNetscapeFuncs.invalidaterect, instance,
-        invalidRect);
+    NPN_InvalidateRectUPP func = gNetscapeFuncs.invalidaterect;
+    if(func)
+    {
+        (*func)(instance, invalidRect);
+    }
 }
 
+/******************************************************************************/
 void
 NPN_InvalidateRegion(NPP instance, NPRegion invalidRegion)
 {
-    CallNPN_InvalidateRegionProc(gNetscapeFuncs.invalidateregion, instance,
-        invalidRegion);
+    NPN_InvalidateRegionUPP func = gNetscapeFuncs.invalidateregion;
+    if(func)
+    {
+        (*func)(instance, invalidRegion);
+    }
 }
 
+/******************************************************************************/
 void
 NPN_ForceRedraw(NPP instance)
 {
-    CallNPN_ForceRedrawProc(gNetscapeFuncs.forceredraw, instance);
+    NPN_ForceRedrawUPP func = gNetscapeFuncs.forceredraw;
+    if(func)
+    {
+        (*func)(instance);
+    }
 }
 
+/******************************************************************************/
 void NPN_PushPopupsEnabledState(NPP instance, NPBool enabled)
 {
-    CallNPN_PushPopupsEnabledStateProc(gNetscapeFuncs.pushpopupsenabledstate,
-        instance, enabled);
+    NPN_PushPopupsEnabledStateUPP func = gNetscapeFuncs.pushpopupsenabledstate;
+    if(func)
+    {
+        (*func)(instance, enabled);
+    }
 }
 
+/******************************************************************************/
 void NPN_PopPopupsEnabledState(NPP instance)
 {
-    CallNPN_PopPopupsEnabledStateProc(gNetscapeFuncs.poppopupsenabledstate,
-        instance);
+    NPN_PopPopupsEnabledStateUPP func = gNetscapeFuncs.poppopupsenabledstate;
+    if(func)
+    {
+        (*func)(instance);
+    }
 }
 
+/******************************************************************************/
 NPIdentifier NPN_GetStringIdentifier(const NPUTF8 *name)
 {
-    return CallNPN_GetStringIdentifierProc(gNetscapeFuncs.getstringidentifier,
-                                           name);
+    NPIdentifier retVal = 0;
+    NPN_GetStringIdentifierUPP func = gNetscapeFuncs.getstringidentifier;
+    if(func)
+    {
+        retVal = (*func)(name);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 void NPN_GetStringIdentifiers(const NPUTF8 **names, int32_t nameCount,
                               NPIdentifier *identifiers)
 {
-    CallNPN_GetStringIdentifiersProc(gNetscapeFuncs.getstringidentifiers,
-        names, nameCount, identifiers);
+    NPN_GetStringIdentifiersUPP func = gNetscapeFuncs.getstringidentifiers;
+    if(func)
+    {
+        (*func)(names, nameCount, identifiers);
+    }
 }
 
+/******************************************************************************/
 NPIdentifier NPN_GetIntIdentifier(int32_t intid)
 {
-    return CallNPN_GetIntIdentifierProc(gNetscapeFuncs.getintidentifier, intid);
+    NPIdentifier retVal = 0;
+    NPN_GetIntIdentifierUPP func = gNetscapeFuncs.getintidentifier;
+    if(func)
+    {
+        retVal = (*func)(intid);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_IdentifierIsString(NPIdentifier identifier)
 {
-    return CallNPN_IdentifierIsStringProc(gNetscapeFuncs.identifierisstring,
-        identifier);
+    bool retVal = false;
+    NPN_IdentifierIsStringUPP func = gNetscapeFuncs.identifierisstring;
+    if(func)
+    {
+        retVal = (*func)(identifier);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPUTF8 *NPN_UTF8FromIdentifier(NPIdentifier identifier)
 {
-    return CallNPN_UTF8FromIdentifierProc(gNetscapeFuncs.utf8fromidentifier,
-        identifier);
+    NPUTF8 * retVal = 0;
+    NPN_UTF8FromIdentifierUPP func = gNetscapeFuncs.utf8fromidentifier;
+    if(func)
+    {
+        retVal = (*func)(identifier);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 int32_t NPN_IntFromIdentifier(NPIdentifier identifier)
 {
-    return CallNPN_IntFromIdentifierProc(gNetscapeFuncs.intfromidentifier,
-        identifier);
+    int32_t retVal = 0;
+    NPN_IntFromIdentifierUPP func = gNetscapeFuncs.intfromidentifier;
+    if(func)
+    {
+        retVal = (*func)(identifier);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPObject *NPN_CreateObject(NPP npp, NPClass *aClass)
 {
-    return CallNPN_CreateObjectProc(gNetscapeFuncs.createobject, npp, aClass);
+    NPObject * retVal = 0;
+    NPN_CreateObjectUPP func = gNetscapeFuncs.createobject;
+    if(func)
+    {
+        retVal = (*func)( npp, aClass);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 NPObject *NPN_RetainObject(NPObject *obj)
 {
-    return CallNPN_RetainObjectProc(gNetscapeFuncs.retainobject, obj);
+    NPObject * retVal = 0;
+    NPN_RetainObjectUPP func = gNetscapeFuncs.retainobject;
+    if(func)
+    {
+        retVal = (*func)( obj);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 void NPN_ReleaseObject(NPObject *obj)
 {
-    CallNPN_ReleaseObjectProc(gNetscapeFuncs.releaseobject, obj);
+    NPN_ReleaseObjectUPP func = gNetscapeFuncs.releaseobject;
+    if(func)
+    {
+        (*func)(obj);
+    }
 }
 
+/******************************************************************************/
 bool NPN_Invoke(NPP npp, NPObject* obj, NPIdentifier methodName,
                 const NPVariant *args, uint32_t argCount, NPVariant *result)
 {
-    return CallNPN_InvokeProc(gNetscapeFuncs.invoke, npp, obj, methodName,
-        args, argCount, result);
+    bool retVal = false;
+    NPN_InvokeUPP func = gNetscapeFuncs.invoke;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, methodName,  args, argCount, result);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_InvokeDefault(NPP npp, NPObject* obj, const NPVariant *args,
                        uint32_t argCount, NPVariant *result)
 {
-    return CallNPN_InvokeDefaultProc(gNetscapeFuncs.invokeDefault, npp, obj,
-        args, argCount, result);
+    bool retVal = false;
+    NPN_InvokeDefaultUPP func = gNetscapeFuncs.invokeDefault;
+    if(func)
+    {
+        retVal = (*func)(npp, obj,  args, argCount, result);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_Evaluate(NPP npp, NPObject* obj, NPString *script,
                   NPVariant *result)
 {
-    return CallNPN_EvaluateProc(gNetscapeFuncs.evaluate, npp, obj, script, result);
+    bool retVal = false;
+    NPN_EvaluateUPP func = gNetscapeFuncs.evaluate;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, script, result);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_GetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
                      NPVariant *result)
 {
-    return CallNPN_GetPropertyProc(gNetscapeFuncs.getproperty, npp, obj,
-        propertyName, result);
+    bool retVal = false;
+    NPN_GetPropertyUPP func = gNetscapeFuncs.getproperty;
+    if(func)
+    {
+       retVal = (*func)(npp, obj, propertyName, result);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_SetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
                      const NPVariant *value)
 {
-    return CallNPN_SetPropertyProc(gNetscapeFuncs.setproperty, npp, obj,
-        propertyName, value);
+    bool retVal = false;
+    NPN_SetPropertyUPP func = gNetscapeFuncs.setproperty;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, propertyName, value);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_RemoveProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
 {
-    return CallNPN_RemovePropertyProc(gNetscapeFuncs.removeproperty, npp, obj,
-        propertyName);
+    bool retVal = false;
+    NPN_RemovePropertyUPP func = gNetscapeFuncs.removeproperty;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, propertyName);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_HasProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
 {
-    return CallNPN_HasPropertyProc(gNetscapeFuncs.hasproperty, npp, obj,
-        propertyName);
+    bool retVal = false;
+    NPN_HasPropertyUPP func = gNetscapeFuncs.hasproperty;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, propertyName);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 bool NPN_HasMethod(NPP npp, NPObject* obj, NPIdentifier methodName)
 {
-    return CallNPN_HasMethodProc(gNetscapeFuncs.hasmethod, npp, obj, methodName);
+    bool retVal = false;
+    NPN_HasMethodUPP func = gNetscapeFuncs.hasmethod;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, methodName);
+    }
+    return retVal;
 }
 
+/******************************************************************************/
 void NPN_ReleaseVariantValue(NPVariant *variant)
 {
-    CallNPN_ReleaseVariantValueProc(gNetscapeFuncs.releasevariantvalue, variant);
+    NPN_ReleaseVariantValueUPP func = gNetscapeFuncs.releasevariantvalue;
+    if(func)
+    {
+        (*func)(variant);
+    }
 }
 
+/******************************************************************************/
 void NPN_SetException(NPObject* obj, const NPUTF8 *message)
 {
-    CallNPN_SetExceptionProc(gNetscapeFuncs.setexception, obj, message);
-}
-
-#endif
-
-/***********************************************************************
- *
- * Wrapper functions : Netscape Navigator -> plugin
- *
- * These functions let the plugin developer just create the APIs
- * as documented and defined in npapi.h, without needing to 
- * install those functions in the function table or worry about
- * setting up globals for 68K plugins.
- *
- ***********************************************************************/
-
-NPError
-Private_New(NPMIMEType pluginType, NPP instance, uint16 mode,
-        int16 argc, char* argn[], char* argv[], NPSavedData* saved)
-{
-    NPError ret;
-    PLUGINDEBUGSTR("New");
-    ret = NPP_New(pluginType, instance, mode, argc, argn, argv, saved);
-    return ret; 
-}
-
-NPError
-Private_Destroy(NPP instance, NPSavedData** save)
-{
-    PLUGINDEBUGSTR("Destroy");
-    return NPP_Destroy(instance, save);
-}
-
-NPError
-Private_SetWindow(NPP instance, NPWindow* window)
-{
-    NPError err;
-    PLUGINDEBUGSTR("SetWindow");
-    err = NPP_SetWindow(instance, window);
-    return err;
-}
-
-NPError
-Private_NewStream(NPP instance, NPMIMEType type, NPStream* stream,
-            NPBool seekable, uint16* stype)
-{
-    NPError err;
-    PLUGINDEBUGSTR("NewStream");
-    err = NPP_NewStream(instance, type, stream, seekable, stype);
-    return err;
-}
-
-int32
-Private_WriteReady(NPP instance, NPStream* stream)
-{
-    unsigned int result;
-    PLUGINDEBUGSTR("WriteReady");
-    result = NPP_WriteReady(instance, stream);
-    return result;
-}
-
-int32
-Private_Write(NPP instance, NPStream* stream, int32 offset, int32 len,
-        void* buffer)
-{
-    unsigned int result;
-    PLUGINDEBUGSTR("Write");
-    result = NPP_Write(instance, stream, offset, len, buffer);
-    return result;
-}
-
-void
-Private_StreamAsFile(NPP instance, NPStream* stream, const char* fname)
-{
-    PLUGINDEBUGSTR("StreamAsFile");
-    NPP_StreamAsFile(instance, stream, fname);
-}
-
-
-NPError
-Private_DestroyStream(NPP instance, NPStream* stream, NPError reason)
-{
-    NPError err;
-    PLUGINDEBUGSTR("DestroyStream");
-    err = NPP_DestroyStream(instance, stream, reason);
-    return err;
-}
-
-void
-Private_URLNotify(NPP instance, const char* url,
-                NPReason reason, void* notifyData)
-                
-{
-    PLUGINDEBUGSTR("URLNotify");
-    NPP_URLNotify(instance, url, reason, notifyData);
-}
-
-
-
-void
-Private_Print(NPP instance, NPPrint* platformPrint)
-{
-    PLUGINDEBUGSTR("Print");
-    NPP_Print(instance, platformPrint);
-}
-
-void *
-Private_GetJavaClass(void)
-{
-    /* This is no longer supported in the newer version of mozilla */
-    PLUGINDEBUGSTR("GetJavaClass");
-    return NULL;
+    NPN_SetExceptionUPP func = gNetscapeFuncs.setexception;
+    if(func)
+    {
+        (*func)(obj, message);
+    }
 }
 
 /*********************************************************************** 
@@ -519,13 +621,13 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
 {
     NPError err = NPERR_NO_ERROR;
 
-    PLUGINDEBUGSTR("NP_Initialize");
-    
     /* validate input parameters */
 
     if ((nsTable == NULL) || (pluginFuncs == NULL))
+    {
         err = NPERR_INVALID_FUNCTABLE_ERROR;
-    
+    }
+
     /*
      * Check the major version passed in Netscape's function table.
      * We won't load if the major version is newer than what we expect.
@@ -535,18 +637,29 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
      *
      */
 
-    if (err == NPERR_NO_ERROR) {
+    if (err == NPERR_NO_ERROR) 
+    {
         if ((nsTable->version >> 8) > NP_VERSION_MAJOR)
+        {
             err = NPERR_INCOMPATIBLE_VERSION_ERROR;
-        if (nsTable->size < sizeof(NPNetscapeFuncs))
-/*        if (nsTable->size < ((char *)&nsTable->setvalue - (char *)nsTable)) */
+        }
+
+        if (nsTable->size < ((char *)&nsTable->getvalue - (char *)nsTable))
+        {
             err = NPERR_INVALID_FUNCTABLE_ERROR;
-        if (pluginFuncs->size < ((char*)&pluginFuncs->getvalue - (char*)pluginFuncs))    
+        }
+
+        if (pluginFuncs->size < ((char*)&pluginFuncs->getvalue - (char*)pluginFuncs))
+        {
             err = NPERR_INVALID_FUNCTABLE_ERROR;
+        }
     }
         
     
-    if (err == NPERR_NO_ERROR) {
+    if (err == NPERR_NO_ERROR) 
+    {
+        /* Zero everything */
+        memset(&gNetscapeFuncs, 0, sizeof(gNetscapeFuncs));
         /*
          * Copy all the fields of Netscape function table into our
          * copy so we can call back into Netscape later.  Note that
@@ -570,20 +683,26 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
         gNetscapeFuncs.memflush      = nsTable->memflush;
         gNetscapeFuncs.reloadplugins = nsTable->reloadplugins;
 
-        gNetscapeFuncs.getJavaEnv    = NULL;   /* No longer supported */
-        gNetscapeFuncs.getJavaPeer   = NULL;   /* No longer supported */
-
         gNetscapeFuncs.posturlnotify = nsTable->posturlnotify;
         gNetscapeFuncs.getvalue      = nsTable->getvalue;
 
-#if 0 /* Peter Leese - new stuff commented out for now */
-        gNetscapeFuncs.setvalue      = nsTable->setvalue;
+        /* Version 0.9?? */
+        if (nsTable->size >= ((char *)&nsTable->setvalue - (char *)nsTable))
+        {
+            gNetscapeFuncs.setvalue  = nsTable->setvalue;
+        }
 
+        /* Version 0.11 */
+        if (nsTable->size >= ((char *)&nsTable->forceredraw - (char *)nsTable))
+        {
+            gNetscapeFuncs.invalidaterect = nsTable->invalidaterect;
+            gNetscapeFuncs.invalidateregion = nsTable->invalidateregion;
+            gNetscapeFuncs.forceredraw = nsTable->forceredraw;
+        }
+
+        /* Version 0.14 */
         if (nsTable->size >= ((char *)&nsTable->setexception - (char *)nsTable))
         {
-          gNetscapeFuncs.invalidaterect = nsTable->invalidaterect;
-          gNetscapeFuncs.invalidateregion = nsTable->invalidateregion;
-          gNetscapeFuncs.forceredraw = nsTable->forceredraw;
           gNetscapeFuncs.getstringidentifier = nsTable->getstringidentifier;
           gNetscapeFuncs.getstringidentifiers = nsTable->getstringidentifiers;
           gNetscapeFuncs.getintidentifier = nsTable->getintidentifier;
@@ -604,42 +723,51 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
           gNetscapeFuncs.releasevariantvalue = nsTable->releasevariantvalue;
           gNetscapeFuncs.setexception = nsTable->setexception;
         }
-         else
-        {
-          gNetscapeFuncs.invalidaterect = NULL;
-          gNetscapeFuncs.invalidateregion = NULL;
-          gNetscapeFuncs.forceredraw = NULL;
-          gNetscapeFuncs.getstringidentifier = NULL;
-          gNetscapeFuncs.getstringidentifiers = NULL;
-          gNetscapeFuncs.getintidentifier = NULL;
-          gNetscapeFuncs.identifierisstring = NULL;
-          gNetscapeFuncs.utf8fromidentifier = NULL;
-          gNetscapeFuncs.intfromidentifier = NULL;
-          gNetscapeFuncs.createobject = NULL;
-          gNetscapeFuncs.retainobject = NULL;
-          gNetscapeFuncs.releaseobject = NULL;
-          gNetscapeFuncs.invoke = NULL;
-          gNetscapeFuncs.invokeDefault = NULL;
-          gNetscapeFuncs.evaluate = NULL;
-          gNetscapeFuncs.getproperty = NULL;
-          gNetscapeFuncs.setproperty = NULL;
-          gNetscapeFuncs.removeproperty = NULL;
-          gNetscapeFuncs.hasproperty = NULL;
-          gNetscapeFuncs.releasevariantvalue = NULL;
-          gNetscapeFuncs.setexception = NULL;
-        }
+
+        /* Version 0.16 */
         if (nsTable->size >=
             ((char *)&nsTable->poppopupsenabledstate - (char *)nsTable))
         {
           gNetscapeFuncs.pushpopupsenabledstate = nsTable->pushpopupsenabledstate;
           gNetscapeFuncs.poppopupsenabledstate  = nsTable->poppopupsenabledstate;
         }
-         else
+
+        /* Version 0.18 */
+        if (nsTable->size >=
+            ((char *)&nsTable->enumerate - (char *)nsTable))
         {
-          gNetscapeFuncs.pushpopupsenabledstate = NULL;
-          gNetscapeFuncs.poppopupsenabledstate  = NULL;
+          gNetscapeFuncs.enumerate = nsTable->enumerate;
         }
-#endif
+
+        /* Version 0.19 */
+        if (nsTable->size >=
+            ((char *)&nsTable->pluginthreadasynccall - (char *)nsTable))
+        {
+          gNetscapeFuncs.pluginthreadasynccall = nsTable->pluginthreadasynccall;
+        }
+
+        /* Version 0.19?? */
+        if (nsTable->size >=
+            ((char *)&nsTable->construct - (char *)nsTable))
+        {
+          gNetscapeFuncs.construct = nsTable->construct;
+        }
+
+        /* Version 0.20 */
+        if (nsTable->size >=
+            ((char *)&nsTable->setvalueforurl - (char *)nsTable))
+        {
+          gNetscapeFuncs.getvalueforurl = nsTable->getvalueforurl;
+          gNetscapeFuncs.setvalueforurl = nsTable->setvalueforurl;
+        }
+
+        /* Version 0.21 */
+        if (nsTable->size >=
+            ((char *)&nsTable->getauthenticationinfo - (char *)nsTable))
+        {
+          gNetscapeFuncs.getauthenticationinfo = nsTable->getauthenticationinfo;
+        }
+
 
         /*
          * Set up the plugin function table that Netscape will use to
@@ -648,27 +776,27 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
          * implement.
          */
         pluginFuncs->version    = (NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR;
-        pluginFuncs->newp       = NewNPP_NewProc(Private_New);
-        pluginFuncs->destroy    = NewNPP_DestroyProc(Private_Destroy);
-        pluginFuncs->setwindow  = NewNPP_SetWindowProc(Private_SetWindow);
-        pluginFuncs->newstream  = NewNPP_NewStreamProc(Private_NewStream);
-        pluginFuncs->destroystream = NewNPP_DestroyStreamProc(Private_DestroyStream);
-        pluginFuncs->asfile     = NewNPP_StreamAsFileProc(Private_StreamAsFile);
-        pluginFuncs->writeready = NewNPP_WriteReadyProc(Private_WriteReady);
-        pluginFuncs->write      = NewNPP_WriteProc(Private_Write);
-        pluginFuncs->print      = NewNPP_PrintProc(Private_Print);
-        pluginFuncs->urlnotify  = NewNPP_URLNotifyProc(Private_URLNotify);
-        pluginFuncs->event      = NULL;
+        pluginFuncs->newp          = NPP_New;
+        pluginFuncs->destroy       = NPP_Destroy;
+        pluginFuncs->setwindow     = NPP_SetWindow;
+        pluginFuncs->newstream     = NPP_NewStream;
+        pluginFuncs->destroystream = NPP_DestroyStream;
+        pluginFuncs->asfile        = NPP_StreamAsFile;
+        pluginFuncs->writeready    = NPP_WriteReady;
+        pluginFuncs->write         = NPP_Write;
+        pluginFuncs->print         = NPP_Print;
+        pluginFuncs->event         = NULL;
+        pluginFuncs->urlnotify     = NPP_URLNotify;
 
-        pluginFuncs->javaClass  = Private_GetJavaClass();
+        pluginFuncs->javaClass  = NULL; /* = NPP_GetJavaClass(); deprecated */
 
         /* All this is so we keep backwards compatibility with old browsers! */ 
         if (pluginFuncs->size > ((char*)&pluginFuncs->getvalue - (char*)pluginFuncs))
         {     
-            pluginFuncs->getvalue = NewNPP_GetValueProc(NP_GetValue);
+            pluginFuncs->getvalue = (NPP_GetValueUPP) NPP_GetValue;
             if (pluginFuncs->size > ((char*)&pluginFuncs->setvalue - (char*)pluginFuncs))
             {    
-                pluginFuncs->setvalue = NULL;
+                pluginFuncs->setvalue = NPP_SetValue;
             }   
         }
 
@@ -693,7 +821,6 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
 NPError
 NP_Shutdown(void)
 {
-    PLUGINDEBUGSTR("NP_Shutdown");
     NPP_Shutdown();
     return NPERR_NO_ERROR;
 }
