@@ -57,55 +57,19 @@
 /*----------------------------------------------------------------------*/
 
 #define NP_VERSION_MAJOR 0
-#define NP_VERSION_MINOR 9
-/* #define NP_VERSION_MINOR 19 (Peter Leese - 3.48 is at version 19) */
-
+#define NP_VERSION_MINOR 11
+// #define NP_VERSION_MINOR 22     // Latest is at version 22!
 
+#include "nptypes.h"
 
 /*----------------------------------------------------------------------*/
 /*                   Definition of Basic Types                          */
 /*----------------------------------------------------------------------*/
  
-#ifndef _UINT16
-typedef unsigned short uint16;
-#endif
-
-#ifndef _UINT32
-#    if defined(__alpha) || defined(__amd64__) || defined(__x86_64__) || defined(__ia64)
-typedef unsigned int uint32;
-#    else /* __alpha */
-typedef unsigned long uint32;
-#    endif /* __alpha */
-#endif
-
-#ifndef _INT16
-typedef short int16;
-#endif
-
-#ifndef _INT32
-#    if defined(__alpha) || defined(__amd64__) || defined(__x86_64__) || defined(__ia64)
-typedef int int32;
-#    else /* __alpha */
-typedef long int32;
-#    endif /* __alpha */
-#endif
-
-#ifndef FALSE
-#define FALSE (0)
-#endif
-#ifndef TRUE
-#define TRUE (1)
-#endif
-#ifndef NULL
-#define NULL (0L)
-#endif
-
 typedef unsigned char	NPBool;
-typedef int16			NPError;
-typedef int16			NPReason;
-typedef char*			NPMIMEType;
-
-
+typedef int16_t         NPError;
+typedef int16_t		NPReason;
+typedef char*		NPMIMEType;
 
 /*----------------------------------------------------------------------*/
 /*                   Structures and definitions                         */
@@ -128,8 +92,8 @@ typedef struct _NPStream
   void* pdata;		/* plug-in private data */
   void* ndata;		/* netscape private data */
   const char* url;
-  uint32 end;
-  uint32 lastmodified;
+  uint32_t end;
+  uint32_t lastmodified;
   void* notifyData;
   const  char* headers; /* Response headers from host.
                          * Exists only for >= NPVERS_HAS_RESPONSE_HEADERS.
@@ -146,31 +110,31 @@ typedef struct _NPStream
 
 typedef struct _NPByteRange
 {
-    int32	offset;			/* negative offset means from the end */
-    uint32	length;
+    int32_t	offset;			/* negative offset means from the end */
+    uint32_t	length;
     struct _NPByteRange* next;
 } NPByteRange;
 
 
 typedef struct _NPSavedData
 {
-    int32	len;
+    int32_t	len;
     void*	buf;
 } NPSavedData;
 
 
 typedef struct _NPRect
 {
-    uint16	top;
-    uint16	left;
-    uint16	bottom;
-    uint16	right;
+    uint16_t	top;
+    uint16_t	left;
+    uint16_t	bottom;
+    uint16_t	right;
 } NPRect;
 
 typedef struct _NPSize 
 { 
-  int32 width; 
-  int32 height; 
+  int32_t width; 
+  int32_t height; 
 } NPSize; 
 
 #ifdef XP_UNIX
@@ -190,12 +154,12 @@ enum {
 
 typedef struct
 {
-    int32		type;
+    int32_t		type;
 } NPAnyCallbackStruct;
 
 typedef struct
 {
-    int32			type;
+    int32_t			type;
     Display*		display;
     Visual*			visual;
     Colormap		colormap;
@@ -204,7 +168,7 @@ typedef struct
 
 typedef struct
 {
-  int32 type;
+  int32_t type;
   FILE* fp;
 } NPPrintCallbackStruct;
 
@@ -229,22 +193,14 @@ typedef struct
  *   gcc 3.x generated vtables on UNIX and OSX are incompatible with 
  *   previous compilers.
  */
-#if (defined (XP_UNIX) && defined(__GNUC__) && (__GNUC__ >= 3))
+#if (defined(XP_UNIX) && defined(__GNUC__) && (__GNUC__ >= 3))
 #define _NP_ABI_MIXIN_FOR_GCC3 NP_ABI_GCC3_MASK
 #else
 #define _NP_ABI_MIXIN_FOR_GCC3 0
 #endif
 
-
+#ifdef XP_MACOSX
 #define NP_ABI_MACHO_MASK 0x01000000
-/*
- *   On OSX, the Mach-O executable format is significantly
- *   different than CFM. In addition to having a different
- *   C++ ABI, it also has has different C calling convention.
- *   You must use glue code when calling between CFM and
- *   Mach-O C functions. 
- */
-#if (defined(TARGET_RT_MAC_MACHO))
 #define _NP_ABI_MIXIN_FOR_MACHO NP_ABI_MACHO_MASK
 #else
 #define _NP_ABI_MIXIN_FOR_MACHO 0
@@ -285,7 +241,15 @@ typedef enum {
    * NPN_MemAlloc() to allocate memory for the string data. Introduced
    * in Mozilla 1.8b2 (NPAPI minor version 15).
    */
-  NPPVformValue = 16
+  NPPVformValue = 16,
+
+  NPPVpluginUrlRequestsDisplayedBool = 17,
+
+  /* Checks if the plugin is interested in receiving the http body of
+   * all http requests (including failed ones, http status != 200).
+   */
+  NPPVpluginWantsAllNetworkStreams = 18
+
 } NPPVariable;
 
 /*
@@ -312,10 +276,15 @@ typedef enum {
   /* Get the NPObject wrapper for the plugins DOM element. */
   NPNVPluginElementNPObject = 16,
 
-  NPNVSupportsWindowless = 17
+  NPNVSupportsWindowless = 17,
 
+  NPNVprivateModeBool = 18
 } NPNVariable;
 
+typedef enum {
+  NPNURLVCookie = 501,
+  NPNURLVProxy
+} NPNURLVariable;
 /*
  * The type of Tookkit the widgets use
  */
@@ -336,25 +305,25 @@ typedef enum {
 typedef struct _NPWindow 
 {
     void*	window;		/* Platform specific window handle */
-    uint32	x;			/* Position of top left corner relative */
-    uint32	y; 			/*	to a netscape page.					*/
-    uint32	width;		/* Maximum window size */
-    uint32	height;
+    int32_t	x;		/* Position of top left corner relative */
+    int32_t	y;		/* to a netscape page. */
+    uint32_t	width;		/* Maximum window size */
+    uint32_t	height;
     NPRect	clipRect;	/* Clipping rectangle in port coordinates */
-						/* Used by MAC only.                      */
-#ifdef XP_UNIX
+				/* Used by MAC only. */
+#if defined(XP_UNIX)
     void *	ws_info;	/* Platform-dependent additonal data */
 #endif /* XP_UNIX */
+  NPWindowType type; /* Is this a window or a drawable? */
 } NPWindow;
 
 
 typedef struct _NPFullPrint
 {
-    NPBool	pluginPrinted;	/* Set TRUE if plugin handled fullscreen */
-							/*	printing							 */
-    NPBool	printOne;		/* TRUE if plugin should print one copy  */
-							/*	to default printer					 */
-    void*	platformPrint;	/* Platform-specific printing info */
+    NPBool	pluginPrinted;/* Set TRUE if plugin handled fullscreen printing */
+    NPBool	printOne;      /* TRUE if plugin should print one copy to default
+                                  printer */
+    void*	platformPrint; /* Platform-specific printing info */
 } NPFullPrint;
 
 typedef struct _NPEmbedPrint
@@ -365,7 +334,7 @@ typedef struct _NPEmbedPrint
 
 typedef struct _NPPrint
 {
-    uint16	mode;						/* NP_FULL or NP_EMBED */
+    uint16_t	mode;						/* NP_FULL or NP_EMBED */
     union
     {
 		NPFullPrint		fullPrint;		/* if mode is NP_FULL */
@@ -448,7 +417,6 @@ typedef void *NPRegion;
 #define NPVERS_HAS_STREAMOUTPUT             8
 #define NPVERS_HAS_NOTIFICATION             9
 #define NPVERS_HAS_LIVECONNECT              9
-#define NPVERS_WIN16_HAS_LIVECONNECT        9
 #define NPVERS_68K_HAS_LIVECONNECT          11
 #define NPVERS_HAS_WINDOWLESS               11
 #define NPVERS_HAS_XPCONNECT_SCRIPTING      13
@@ -458,13 +426,15 @@ typedef void *NPRegion;
 #define NPVERS_HAS_RESPONSE_HEADERS         17
 #define NPVERS_HAS_NPOBJECT_ENUM            18
 #define NPVERS_HAS_PLUGIN_THREAD_ASYNC_CALL 19
+#define NPVERS_HAS_ALL_NETWORK_STREAMS      20
+#define NPVERS_HAS_URL_AND_AUTH_INFO        21
 
 /*----------------------------------------------------------------------*/
 /*                        Function Prototypes                           */
 /*----------------------------------------------------------------------*/
 
-#if defined(_WINDOWS) && !defined(WIN32)
-#define NP_LOADDS  _loadds
+#if defined(__OS2__)
+#define NP_LOADDS  _System
 #else
 #define NP_LOADDS
 #endif
@@ -473,77 +443,94 @@ typedef void *NPRegion;
 extern "C" {
 #endif
 
-/*
- * NPP_* functions are provided by the plugin and called by the navigator.
- */
+/* NPP_* functions are provided by the plugin and called by the navigator. */
 
 #ifdef XP_UNIX
 char* NPP_GetMIMEDescription(void);
-#endif /* XP_UNIX */
+#endif
 
 NPError NP_LOADDS NPP_Initialize(void);
 void    NP_LOADDS NPP_Shutdown(void);
 NPError NP_LOADDS NPP_New(NPMIMEType pluginType, NPP instance,
-                          uint16 mode, int16 argc, char* argn[],
+                          uint16_t mode, int16_t argc, char* argn[],
                           char* argv[], NPSavedData* saved);
 NPError NP_LOADDS NPP_Destroy(NPP instance, NPSavedData** save);
 NPError NP_LOADDS NPP_SetWindow(NPP instance, NPWindow* window);
 NPError NP_LOADDS NPP_NewStream(NPP instance, NPMIMEType type,
                                 NPStream* stream, NPBool seekable,
-                                uint16* stype);
+                                uint16_t* stype);
 NPError NP_LOADDS NPP_DestroyStream(NPP instance, NPStream* stream,
                                     NPReason reason);
-int32   NP_LOADDS NPP_WriteReady(NPP instance, NPStream* stream);
-int32   NP_LOADDS NPP_Write(NPP instance, NPStream* stream, int32 offset,
-                            int32 len, void* buffer);
+int32_t NP_LOADDS NPP_WriteReady(NPP instance, NPStream* stream);
+int32_t NP_LOADDS NPP_Write(NPP instance, NPStream* stream, int32_t offset,
+                            int32_t len, void* buffer);
 void    NP_LOADDS NPP_StreamAsFile(NPP instance, NPStream* stream,
                                    const char* fname);
 void    NP_LOADDS NPP_Print(NPP instance, NPPrint* platformPrint);
-int16   NP_LOADDS NPP_HandleEvent(NPP instance, void* event);
+int16_t NP_LOADDS NPP_HandleEvent(NPP instance, void* event);
 void    NP_LOADDS NPP_URLNotify(NPP instance, const char* url,
                                 NPReason reason, void* notifyData);
 
 NPError NP_LOADDS NPP_GetValue(void * instance, NPPVariable variable, void *value);
 NPError NP_LOADDS NPP_SetValue(NPP instance, NPNVariable variable, void *value);
 
-/*
- * NPN_* functions are provided by the navigator and called by the plugin.
- */
-void    NP_LOADDS NPN_Version(int* plugin_major, int* plugin_minor,
+/* NPN_* functions are provided by the navigator and called by the plugin. */
+
+void        NP_LOADDS NPN_Version(int* plugin_major, int* plugin_minor,
                               int* netscape_major, int* netscape_minor);
-NPError NP_LOADDS NPN_GetURLNotify(NPP instance, const char* url,
+NPError     NP_LOADDS NPN_GetURLNotify(NPP instance, const char* url,
                                    const char* target, void* notifyData);
-NPError NP_LOADDS NPN_GetURL(NPP instance, const char* url,
+NPError     NP_LOADDS NPN_GetURL(NPP instance, const char* url,
                              const char* target);
-NPError NP_LOADDS NPN_PostURLNotify(NPP instance, const char* url,
-                                    const char* target, uint32 len,
+NPError     NP_LOADDS NPN_PostURLNotify(NPP instance, const char* url,
+                                    const char* target, uint32_t len,
                                     const char* buf, NPBool file,
                                     void* notifyData);
-NPError NP_LOADDS NPN_PostURL(NPP instance, const char* url,
-                              const char* target, uint32 len,
+NPError     NP_LOADDS NPN_PostURL(NPP instance, const char* url,
+                              const char* target, uint32_t len,
                               const char* buf, NPBool file);
-NPError NP_LOADDS NPN_RequestRead(NPStream* stream, NPByteRange* rangeList);
-NPError NP_LOADDS NPN_NewStream(NPP instance, NPMIMEType type,
+NPError     NP_LOADDS NPN_RequestRead(NPStream* stream, NPByteRange* rangeList);
+NPError     NP_LOADDS NPN_NewStream(NPP instance, NPMIMEType type,
                                 const char* target, NPStream** stream);
-int32   NP_LOADDS NPN_Write(NPP instance, NPStream* stream, int32 len, void* buffer);
-NPError NP_LOADDS NPN_DestroyStream(NPP instance, NPStream* stream, NPReason reason);
-void    NP_LOADDS NPN_Status(NPP instance, const char* message);
+int32_t     NP_LOADDS NPN_Write(NPP instance, NPStream* stream, int32_t len, 
+                                void* buffer);
+NPError     NP_LOADDS NPN_DestroyStream(NPP instance, NPStream* stream, 
+                                        NPReason reason);
+void        NP_LOADDS NPN_Status(NPP instance, const char* message);
 const char* NP_LOADDS	NPN_UserAgent(NPP instance);
-void*   NP_LOADDS NPN_MemAlloc(uint32 size);
-void    NP_LOADDS NPN_MemFree(void* ptr);
-uint32  NP_LOADDS NPN_MemFlush(uint32 size);
-void    NP_LOADDS NPN_ReloadPlugins(NPBool reloadPages);
+void*       NP_LOADDS NPN_MemAlloc(uint32_t size);
+void        NP_LOADDS NPN_MemFree(void* ptr);
+uint32_t    NP_LOADDS NPN_MemFlush(uint32_t size);
+void        NP_LOADDS NPN_ReloadPlugins(NPBool reloadPages);
 
-NPError NP_LOADDS NPN_GetValue(NPP instance, NPNVariable variable, void *value);
-NPError NP_LOADDS NPN_SetValue(NPP instance, NPPVariable variable, void *value);
-void    NP_LOADDS NPN_InvalidateRect(NPP instance, NPRect *invalidRect);
-void    NP_LOADDS NPN_InvalidateRegion(NPP instance, NPRegion invalidRegion);
-void    NP_LOADDS NPN_ForceRedraw(NPP instance);
-void    NP_LOADDS NPN_PushPopupsEnabledState(NPP instance, NPBool enabled);
-void    NP_LOADDS NPN_PopPopupsEnabledState(NPP instance);
-void    NP_LOADDS NPN_PluginThreadAsyncCall(NPP instance,
+NPError     NP_LOADDS NPN_GetValue(NPP instance, NPNVariable variable, 
+                                   void *value);
+NPError     NP_LOADDS NPN_SetValue(NPP instance, NPPVariable variable, 
+                                   void *value);
+void        NP_LOADDS NPN_InvalidateRect(NPP instance, NPRect *invalidRect);
+void        NP_LOADDS NPN_InvalidateRegion(NPP instance, 
+                                           NPRegion invalidRegion);
+void        NP_LOADDS NPN_ForceRedraw(NPP instance);
+void        NP_LOADDS NPN_PushPopupsEnabledState(NPP instance, NPBool enabled);
+void        NP_LOADDS NPN_PopPopupsEnabledState(NPP instance);
+void        NP_LOADDS NPN_PluginThreadAsyncCall(NPP instance,
                                             void (*func) (void *),
                                             void *userData);
+NPError     NP_LOADDS NPN_GetValueForURL(NPP instance, NPNURLVariable variable,
+                                         const char *url, char **value,
+                                         uint32_t *len);
+NPError     NP_LOADDS NPN_SetValueForURL(NPP instance, NPNURLVariable variable,
+                                         const char *url, const char *value,
+                                         uint32_t len);
+NPError     NP_LOADDS NPN_GetAuthenticationInfo(NPP instance,
+                                                const char *protocol,
+                                                const char *host, int32_t port,
+                                                const char *scheme,
+                                                const char *realm,
+                                                char **username, uint32_t *ulen,
+                                                char **password,
+                                                uint32_t *plen);
+
 
 #ifdef __cplusplus
 }  /* end extern "C" */
